//****************************************************************************
//	Copyright (c)1993 KNOPS EDV-Anlagen Gerd Knops
//
//	Project	: GKLibrary
//	File	: GKIOCtrl
//	Author	: Gerd Knops
//	Date	: 930907
//
//****************************************************************************
//
//	History:
//	930907 Creation of file
//
//****************************************************************************
//****************************************************************************
// Constants
//****************************************************************************

	#define GK_SHOWFILE		1		// if File should be preselected
	#define GK_MULTIPLE		2		// (open) if multiple files are allowed
	#define GK_DIRECTORIES	4		// (open) if to select just a directory
	#define GK_PACKAGES		8		// if to show packages as a file

@interface GKIOCtrl:Object
{
	char	*pathType;
	char	*path;
	char	*file;
}
//****************************************************************************
// <PRIVATE>
// class methods for internal use
//****************************************************************************
+ initialize;
+ instanceForType:(const char *)pType;
//****************************************************************************
// <PUBLIC>
// class methods
//****************************************************************************
+ (const char *)homePath;
+ (const char *)privatePath;
+ (char *)homeLibrary:(char *)returnPath;
+ (char *)localLibrary:(char *)returnPath;
+ (char *)nextLibrary:(char *)returnPath;
+ (const char *)relativePathFor:(const char *)thePath;
// <PRIVATE>
+ (const char *)classPathFor:theClass;
// <PUBLIC>
+ (const char *)getOpenPathFor	:(const char *const *)fileExtensions
						title	:(const char *)title
						type	:(const char *)type
						flags	:(short)flags;
+ (const char *)getOpenPathFor	:(const char *const *)fileExtensions
						title	:(const char *)title
						type	:(const char *)type
						flags	:(short)flags
				accessoryView	:accView;
+ (const char *)getSavePathFor	:(const char *)fileExtension
						title	:(const char *)title
						type	:(const char *)type
						flags	:(short)flags;
+ (const char *)getSavePathFor	:(const char *)fileExtension
						title	:(const char *)title
						type	:(const char *)type
						flags	:(short)flags
				accessoryView	:accView;
+ (const char *)getPathFor:(const char *)type;
+ (BOOL)copyFile:(const char *)srcFile to:(const char *)destFile;
+ (BOOL)copyPathToPrivate:(const char *)path;
+ (BOOL)copyPath:(const char *)source to:(const char *)destination;
+ (BOOL)mkdirs:(const char *)mkpath;
+ (BOOL)deletePath:(const char *)delPath;
//****************************************************************************
// convenience methods
//****************************************************************************
+ loadErrorFor:(const char *)theFile;
+ saveErrorFor:(const char *)theFile;
+ createErrorFor:(const char *)theFile;
+ executeErrorFor:(const char *)theFile;
//****************************************************************************
// <PRIVATE>
// factory methods
//****************************************************************************
- initForType:(const char *)pType;
//****************************************************************************
// implementation methods
//****************************************************************************
- (const char *)pathType;
- (const char *)path;
- setPath:(const char *)thePath;
- (const char *)file;
- setFile:(const char *)theFile;
- (const char *)getOpenPathFor	:(const char * const *)fileExtensions
						title	:(const char *)title
						flags	:(short)flags
				accessoryView	:accView;
- (const char *)getSavePathFor	:(const char *)fileExtension
						title	:(const char *)title
						flags	:(short)flag
				accessoryView	:accView;
//****************************************************************************
// Archiver related methods
//****************************************************************************
- write:(NXTypedStream *)theStream;
- read:(NXTypedStream *)theStream;
// <PUBLIC>
@end
// <PRIVATE>
//============================================================================
//
//	BITart CATEGORY for Application
//
//============================================================================
@interface Application(GKIOCtrl)
//****************************************************************************
// new methods
//****************************************************************************
- (BOOL)isAutoupdate;
@end
//*************************************************************************EOF