//**************************************************************************
//	Copyright (c)1993 KNOPS EDV-Anlagen Gerd Knops
//
//	Project	: GKLibrary
//	File	: GKModule
//	Author	: Gerd Knops
//	Date	: 931120
//
//**************************************************************************
#import <stdio.h>
//**************************************************************************
@interface GKModule:Object
{
	char	*name;			// The Modules name, that is the name of
							// the bundle without extension, or, in
							// case of the appModule, the name off the
							// application.
	
	id		bundle;			// The bundle for this module.
	id		infoStrings;	// The string table, that holds the
							// module relevant data.
	id		localStrings;	// The bundles basic localization string table.
	id		menuItemList;	// A list containing menuCells that belong to
							// this module.
	id		image;			// The bitimage for this module.
	
	id		infoPanel;		// The modules info panel, if any.
	
	char	**manualPaths;	// A list of items and help paths for those.
	
	BOOL	testedForImage;	// An internal variable.
	BOOL	isActive;		// An internal variable.
}
//**************************************************************************
// <PRIVATE>
// Class methods
//**************************************************************************
+ setUpdateServices:(BOOL)flag;
+ (BOOL)mustUpdateServices;
// <PUBLIC>
//**************************************************************************
// Factory methods
//**************************************************************************
- initAppModule;
- initForDirectory:(const char *)thePath;
//**************************************************************************
// Request methods
//**************************************************************************
- (const char *)name;
- bundle;
- localStrings;
- infoStrings;
- image;
- loadImage:(const char *)theName;
// <PRIVATE>
- menuItemList;
- (const char *)manualPathForItem:(const char *)theItem;
// <PUBLIC>
//**************************************************************************
// Methods called from outside
//**************************************************************************
// <PRIVATE>
- (BOOL)active;
- setActive:(BOOL)flag;
// <PUBLIC>
- loadNib:(const char *)nibFileName owner:owner;
- loadNib:(const char *)nibFileName owner:owner fromZone:(NXZone *)zone;
- setupMenuItem:(int)number target:newTarget selector:(SEL)newSelector;
//**************************************************************************
// <PRIVATE>
// Integration into UI
//**************************************************************************
- activate;
- addToUI;
//**************************************************************************
// Integration into Menu
//**************************************************************************
- addToMenu;
- menuTargetSubstitute:sender;
- addMenuItem	:(const char *)thePath
		tag		:(int)theTag
		target	:theTarget
		selector:(SEL)theSelector
		key		:(char)theKey;
//**************************************************************************
// Integration into Services
//**************************************************************************
- checkForServices;
- (BOOL)isProviderForService:(const char *)serviceMessage;
- addServiceDescriptionsTo:(FILE *)theFile;
//**************************************************************************
// Other methods
//**************************************************************************
- load;
- (BOOL)isLoaded;
- showInfoPanel:sender;
//**************************************************************************
// Delegate Methods for Info Panel
//**************************************************************************
- windowWillClose:sender;
@end
//**************************************************************************
@interface Menu(GKModule)
//**************************************************************************
// GKModule related methods
//**************************************************************************
- (char *)compressedNameFrom:(const char *)fullName;
- createCellWithPath:(const char *)thePath for:module autoupdate:(BOOL)update;
- (int)findInsertPositionFor:(char *)posInfo;
- (BOOL)menuNameCompare:menuCell:(const char *)compare;
@end
//***********************************************************************EOF