// SCSIsd.m
//
// History :
//		10.19.1992 : created by Max Tardiveau

#import "SCSIsd.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation SCSIsd


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "SCSIsd : getkval : lseek error for %p\n", (void *)offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "SCSIsd : getkval : read error at %p\n", (void *)offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int						i, cellCount;
id						newCell;
int 					kmem_file;
struct nlist			corelist[2];
struct scsi_disk_drive	tmpDrive;
struct scsi_device		tmpDevice;
char					tmpChar[32];

if ([theCell updater])		// It's a device
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

/////// Get list of devices
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "SCSIsd : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_sd_sdd";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) || (corelist[0].n_value == 0))
	{
#ifdef DEBUG
	fprintf(stderr, "SCSIsd : error in nlist().\n");
#endif
	[MyMaster addMessage:"SCSIsd : error in nlist().\n"];
	return self;
	}

for (i = 0; i < SCSI_NTARGETS; i++)
	{
	getkval(kmem_file, corelist[0].n_value+(i*sizeof(struct scsi_disk_drive)),
								sizeof(struct scsi_disk_drive), &tmpDrive);
	if (tmpDrive.sdd_sdp == NULL)
		break;

	getkval(kmem_file, (int)tmpDrive.sdd_sdp, sizeof(struct scsi_device), 
																&tmpDevice);
	if (tmpDevice.sd_target == 7)
		break;
	
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	sprintf(tmpChar, "Target %i", tmpDevice.sd_target);
	[newCell setStringValue:tmpChar];
	[newCell setFreePointer:(void *)tmpDevice.sd_target];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:NO];
	}

[theCell setMasterInstance:self];

if (close(kmem_file) < 0)
	fprintf(stderr, "SCSIsd : error closing kmem_file.\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct scsi_disk_drive	tmpDrive;
struct scsi_device		tmpDevice;
struct inquiry_reply	tmpReply;
int						i, tmpInt, tmpInt2;
char					tmpChar[32];

/////// Find device with same target number
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "SCSIsd : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_sd_sdd";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) || (corelist[0].n_value == 0))
	{
#ifdef DEBUG
	fprintf(stderr, "SCSIsd : error in nlist().\n");
#endif
	[MyMaster addMessage:"SCSIsd : error in nlist().\n"];
	return self;
	}

tmpInt = NO;
for (i = 0; i < SCSI_NTARGETS; i++)
	{
	getkval(kmem_file, corelist[0].n_value+(i*sizeof(struct scsi_disk_drive)),
								sizeof(struct scsi_disk_drive), &tmpDrive);
	if ( ! tmpDrive.sdd_sdp)
		break;
		
	getkval(kmem_file, (int)tmpDrive.sdd_sdp, sizeof(struct scsi_device), 
																&tmpDevice);

	if ((int)tmpDevice.sd_target == (int)[theCell freePointer])
		{
		tmpInt = YES;		// Found it
		break;
		}
	}

if (tmpInt)				// Found it
	{
	[targetNumber setIntValue:(int)tmpDevice.sd_target];
	[lunNumber setIntValue:(int)tmpDevice.sd_lun];
	getkval(kmem_file, (int)tmpDrive.sdd_irp,
								sizeof(struct inquiry_reply), &tmpReply);

	for (i = 0; i < 6; i++)
		[[typeMatrix cellAt:i :0] setState:0];
	[[typeMatrix cellAt:(int)tmpReply.ir_devicetype :0] setState:1];
	[removable setState:tmpReply.ir_removable];
	[typeQualifier setIntValue:tmpReply.ir_typequalifier];
	[ECMA setIntValue:tmpReply.ir_ecmaversion];
	[ANSI setIntValue:tmpReply.ir_ansiversion];
	tmpInt2 = tmpReply.ir_zero3[2] << 16;
	tmpInt2 += tmpReply.ir_zero3[1] << 8;
	tmpInt2 += tmpReply.ir_zero3[0];
	[vendorID setIntValue:tmpInt2];
	strncpy(tmpChar, tmpReply.ir_vendorid, 8);
	tmpChar[8] = '\0';
	[vendorName setStringValue:tmpChar];
	strncpy(tmpChar, tmpReply.ir_productid, 16);
	tmpChar[16] = '\0';
	[productName setStringValue:tmpChar];
	strncpy(tmpChar, tmpReply.ir_revision, 32);
	tmpChar[32] = '\0';
	[revisionLevel setStringValue:tmpChar];
	[dataFormat setIntValue:tmpReply.ir_rspdatafmt];
	}
else
	{
	[MyMaster addMessage:"SCSIsd : Could not find target.\n"];
	[targetNumber setStringValue:"Error"];
	[lunNumber setStringValue:"Error"];
	for (i = 0; i <= DEVTYPE_READONLY; i++)
		[[typeMatrix cellAt:i :0] setIntValue:0];
	[removable setState:0];
	[typeQualifier setStringValue:"Error"];
	[ECMA setStringValue:"Error"];
	[ANSI setStringValue:"Error"];
	[vendorID setStringValue:"Error"];
	[vendorName setStringValue:"Error"];
	[productName setStringValue:"Error"];
	[revisionLevel setStringValue:"Error"];
	[dataFormat setStringValue:"Error"];
	}

if (close(kmem_file) < 0)
	fprintf(stderr, "SCSIsd : error closing kmem_file (2).\n");

return self;
}

@end
