// SCSIsdstatus.m
//
// History :
//		10.19.1992 : created by Max Tardiveau

#import "SCSIsdstatus.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation SCSIsdstatus


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : lseek error for %p\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : read error at %p\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct scsi_disk_drive	tmpDrive;
struct scsi_device		tmpDevice;
int						i, tmpInt;

/////// Find device with same target number
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "SCSIsdvolume : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_sd_sdd";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) || (corelist[0].n_value == 0))
	{
#ifdef DEBUG
	fprintf(stderr, "SCSIsdvolume : error in nlist().\n");
#endif
	[MyMaster addMessage:"SCSIsdvolume : error in nlist().\n"];
	return self;
	}

tmpInt = NO;
for (i = 0; i < SCSI_NTARGETS; i++)
	{
	getkval(kmem_file, corelist[0].n_value+(i*sizeof(struct scsi_disk_drive)),
								sizeof(struct scsi_disk_drive), &tmpDrive);
	if ( ! tmpDrive.sdd_sdp)
		break;
		
	getkval(kmem_file, (int)tmpDrive.sdd_sdp, sizeof(struct scsi_device), 
																&tmpDevice);

	if ((int)tmpDevice.sd_target == (int)[theCell freePointer])
		{
		tmpInt = YES;		// Found it
		break;
		}
	}

if (tmpInt)				// Found it
	{
	[[statusMatrix cellAt:0 :0] setIntValue:
									~(tmpDrive.sdd_flags & SDF_POLLING)];
	[[statusMatrix cellAt:1 :0] setIntValue:
									(tmpDrive.sdd_flags & SDF_EJECTING)];
	[[statusMatrix cellAt:2 :0] setIntValue:
									(tmpDrive.sdd_flags & SDF_DISCONNECT)];
	[[statusMatrix cellAt:3 :0] setIntValue:
									(tmpDrive.sdd_flags & SDF_GETLABEL)];
	
	[[actionMatrix cellAt:tmpDrive.sdd_state :0] setState:1];
	
	[busyRetries setIntValue:tmpDrive.sdd_busy_retry];
	[grossRetries setIntValue:tmpDrive.sdd_gross_retry];
	[mediaRetries setIntValue:tmpDrive.sdd_med_retry];
	[WFRRetries setIntValue:tmpDrive.sdd_rdy_retry];
	[maxRetries setIntValue:tmpDrive.sdd_med_retry_max];
	}
else
	{
	for (i = 0; i < 4; i++)
		[[statusMatrix cellAt:i :0] setIntValue:0];
	[busyRetries setStringValue:"Error"];
	[grossRetries setStringValue:"Error"];
	[mediaRetries setStringValue:"Error"];
	[WFRRetries setStringValue:"Error"];
	[maxRetries setStringValue:"Error"];
	}

if (close(kmem_file) < 0)
	fprintf(stderr, "SCSIsdvolume : error closing kmem_file (2).\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 10;
}

@end
