// SCSIsdvolume.m
//
// History :
//		10.19.1992 : created by Max Tardiveau

#import "SCSIsdvolume.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation SCSIsdvolume


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : lseek error for %p\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : read error at %p\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct scsi_disk_drive	tmpDrive;
struct scsi_device		tmpDevice;
struct scsi_disk_volume	tmpVolume;
struct capacity_reply	tmpCapacity;
int						i, tmpInt;

/////// Find device with same target number
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "SCSIsdvolume : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_sd_sdd";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) || (corelist[0].n_value == 0))
	{
#ifdef DEBUG
	fprintf(stderr, "SCSIsdvolume : error in nlist().\n");
#endif
	[MyMaster addMessage:"SCSIsdvolume : error in nlist().\n"];
	return self;
	}

tmpInt = NO;
for (i = 0; i < SCSI_NTARGETS; i++)
	{
	getkval(kmem_file, corelist[0].n_value+(i*sizeof(struct scsi_disk_drive)),
								sizeof(struct scsi_disk_drive), &tmpDrive);
	if ( ! tmpDrive.sdd_sdp)
		break;
		
	getkval(kmem_file, (int)tmpDrive.sdd_sdp, sizeof(struct scsi_device), 
																&tmpDevice);

	if ((int)tmpDevice.sd_target == (int)[theCell freePointer])
		{
		tmpInt = YES;		// Found it
		break;
		}
	}

if (tmpInt)				// Found it
	{
	getkval(kmem_file, (int)tmpDrive.sdd_sdvp,
								sizeof(struct scsi_disk_volume), &tmpVolume);
	[volNumber setIntValue:tmpVolume.sdv_volume_num];
	[[statusMatrix cellAt:0 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_NOTAVAIL)];
	[[statusMatrix cellAt:1 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_MOUNTED)];
	[[statusMatrix cellAt:2 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_LABELVALID)];
	[[statusMatrix cellAt:3 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_ALERT)];
	[[statusMatrix cellAt:4 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_NEWVOLUME)];
	[[statusMatrix cellAt:5 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_VALID)];
	[[statusMatrix cellAt:6 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_ACTIVE)];
	[[statusMatrix cellAt:7 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_WAITING)];
	[[statusMatrix cellAt:8 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_FORMATTED)];
	[[statusMatrix cellAt:9 :0] setIntValue:
									(tmpVolume.sdv_flags & SVF_WP)];
	[blocksPerBlock setIntValue:tmpVolume.sdv_bratio];
	[ownerName setStringValue:getpwuid(tmpVolume.sdv_owner)->pw_name];

	getkval(kmem_file, (int)tmpVolume.sdv_crp,
								sizeof(struct capacity_reply), &tmpCapacity);
	[lastBlock setIntValue:tmpCapacity.cr_lastlba];
	[blockLength setIntValue:tmpCapacity.cr_blklen];
	}
else
	{
	[volNumber setStringValue:"Error"];
	for (i = 0; i < 10; i++)
		[[statusMatrix cellAt:i :0] setIntValue:0];
	[blocksPerBlock setStringValue:"Error"];
	[ownerName setStringValue:"Error"];
	[lastBlock setStringValue:"Error"];
	[blockLength setStringValue:"Error"];
	}

if (close(kmem_file) < 0)
	fprintf(stderr, "SCSIsdvolume : error closing kmem_file (2).\n");

return self;
}

@end
