// SCSIststatus.m
//
// History :
//		10.19.1992 : created by Max Tardiveau

#import "SCSIststatus.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation SCSIststatus


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : lseek error for %p\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "SCSIsdvolume : getkval : read error at %p\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct scsi_tape_device	tmpDrive;
struct scsi_device		tmpDevice;
struct capacity_reply	tmpReply;
int						i, tmpInt;

/////// Find device with same target number
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "SCSIsdvolume : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_st_std";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) || (corelist[0].n_value == 0))
	{
#ifdef DEBUG
	fprintf(stderr, "SCSIsdvolume : error in nlist().\n");
#endif
	[MyMaster addMessage:"SCSIsdvolume : error in nlist().\n"];
	return self;
	}

tmpInt = NO;
for (i = 0; i < SCSI_NTARGETS; i++)
	{
	getkval(kmem_file, corelist[0].n_value+(i*sizeof(struct scsi_tape_device)),
								sizeof(struct scsi_tape_device), &tmpDrive);
	if ( ! tmpDrive.std_sdp)
		break;
		
	getkval(kmem_file, (int)tmpDrive.std_sdp, sizeof(struct scsi_device), 
																&tmpDevice);

	if ((int)tmpDevice.sd_target == (int)[theCell freePointer])
		{
		tmpInt = YES;		// Found it
		break;
		}
	}

if (tmpInt)				// Found it
	{
	getkval(kmem_file, (int)tmpDrive.std_crp, sizeof(struct capacity_reply), 
																&tmpReply);
	[lastBlock setIntValue:tmpReply.cr_lastlba];
	[blockLength setIntValue:tmpReply.cr_blklen];
	[[statusMatrix cellAt:0 :0] setIntValue:
									(tmpDrive.std_flags & STF_POLL_IP)];
	[[statusMatrix cellAt:1 :0] setIntValue:
									(tmpDrive.std_flags & STF_ERPVALID)];
	[[statusMatrix cellAt:2 :0] setIntValue:
									(tmpDrive.std_flags & STF_FIXED)];
	[[statusMatrix cellAt:3 :0] setIntValue:
									(tmpDrive.std_flags & STF_WRITE)];
	[[statusMatrix cellAt:4 :0] setIntValue:
									(tmpDrive.std_flags & STF_OPEN)];
	[[statusMatrix cellAt:5 :0] setIntValue:
									(tmpDrive.std_flags & STF_SIL)];
	
	for (i = 0; i < 3; i++)
		[[actionMatrix cellAt:i :0] setState:0];
	[[actionMatrix cellAt:tmpDrive.std_state :0] setState:1];
	
	[waitLimit setIntValue:tmpDrive.std_waitlimit];
	[waitCount setIntValue:tmpDrive.std_waitcnt];
	if (tmpDrive.std_flags & STF_FIXED)
		[fixedSize setIntValue:tmpDrive.std_blocksize];
	else
		[fixedSize setStringValue:"N/A"];
	[retriesCount setIntValue:tmpDrive.std_tries];
	}
else
	{
	for (i = 0; i < 6; i++)
		[[statusMatrix cellAt:i :0] setIntValue:0];
	[waitLimit setStringValue:"Error"];
	[waitCount setStringValue:"Error"];
	[fixedSize setStringValue:"Error"];
	[retriesCount setStringValue:"Error"];
	[MyMaster addMessage:"SCSIststatus : could not find target.\n"];
	}

if (close(kmem_file) < 0)
	fprintf(stderr, "SCSIststatus : error closing kmem_file (2).\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 10;
}

@end
