// KernLoaderServers.m
//
// History :
//		10.24.1992 : created by Max Tardiveau

#import "KernLoaderServers.h"


@implementation KernLoaderServers


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int					i, cellCount;
id					newCell;
kern_return_t		result;
port_t				loader_port;
server_name_array_t	server_names;
unsigned int		count;

if ([theCell updater])		// It's a server
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:NULL 
												string:[theCell freeString]];
	return self;
	}

result = kern_loader_look_up(&loader_port);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:"KernLoaderServers: error in kern_loader_look_up.\n"];
    return self;
	}

result = kern_loader_server_list(loader_port, &server_names, &count);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:
					"KernLoaderServers: error in kern_loader_server_list.\n"];
    return self;
	}

for (i = 0; i < count; i++)
	{
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	[newCell setStringValue:server_names[i]];
	[newCell setFreeString:server_names[i]];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:NO];
	}

[theCell setMasterInstance:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t				result;
port_t						loader_port;
server_state_t				server_state;
vm_address_t				load_addr;
vm_size_t					load_size;
server_reloc_t				relocatable, loadable;
port_name_array_t			port_list;
unsigned int				count;
port_name_string_array_t	port_names;
boolean_array_t				advertised;
task_t						kernel_port;
int							i;
char						tmpChar[32];

result = kern_loader_look_up(&loader_port);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:
					"KernLoaderServers : error in kern_loader_look_up (2).\n"];
    return self;
	}

result=task_by_unix_pid(task_self(), 0, &kernel_port);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:"KernLoaderServers : error in task_by_unix_pid.\n"];
    return self;
	}
	
result= kern_loader_server_info(loader_port, kernel_port, [theCell freeString], 
	&server_state, &load_addr, &load_size, relocatable, loadable, &port_list, 
	&count, &port_names, &count, &advertised, &count);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:
					"KernLoaderServers : error in kern_loader_server_info.\n"];
    return self;
	}

[serverName setStringValue:[theCell freeString]];
for (i = 0; i < (Deallocated + 1); i++)
	[[stateMatrix cellAt:i :0] setState:0];
[[stateMatrix cellAt:server_state :0] setState:1];
[relocFile setStringValue:relocatable];
[loadFile setStringValue:loadable];
sprintf(tmpChar, "%p", (void *)load_addr);
[loadAddress setStringValue:tmpChar];
[loadSize setIntValue:load_size];
[numPorts setIntValue:count];

return self;
}


@end
