// NetIfAddresses.m
//
// History :
//		11.01.1992 : created by Max Tardiveau

#import "NetIfAddresses.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation NetIfAddresses


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static int getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetIfAddresses : getkval : lseek error for %x\n", offset);
	return 0;
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetIfAddresses : getkval : read error at %x\n", offset);
	return 0;
	}

return 1;
}

////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int				cellCount;
id				newCell;
int 			kmem_file;
int				kmem_result;
struct nlist	corelist[2];
struct ifnet	*ifnetPtr, IfNet;
struct ifaddr	*ifaddrPtr, IfAddr;
char			tmpChar[16];
int				i, foundIt = NO;

if ([theCell updater])		// It's an address
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

/////// Get list of interfaces
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetIfAddresses : error opening KMEM.\n"];
	return self;
	}

corelist[0].n_un.n_name = "_ifnet";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"NetIfAddresses : error in nlist().\n"];
	close(kmem_file);
	return self;
	}

kmem_result = getkval(kmem_file,corelist[0].n_value, sizeof(struct ifnet *), 
																	&ifnetPtr);
if (kmem_result == 0)
	{
	[MyMaster addMessage:"NetIfAddresses : error reading kernel.\n"];
	return self;
	}

do			// This assumes there is at least one interface (!)
	{
	kmem_result = getkval(kmem_file, (int)ifnetPtr, sizeof(struct ifnet), 
																	&IfNet);
	if (kmem_result == 0)
		{
		[MyMaster addMessage:"NetIfAddresses : error reading kernel.\n"];
		return self;
		}
		
	if ([theCell freePointer] == ifnetPtr)
		{
		foundIt = YES;
		break;
		}

	if ( ! IfNet.if_next)
		break;
	ifnetPtr = IfNet.if_next;
	}
while (ifnetPtr);

if (foundIt && (IfNet.if_flags & IFF_UP))
	{
	ifaddrPtr = IfNet.if_addrlist;
	i = 1;
	do
		{
		kmem_result = getkval(kmem_file, (int)ifaddrPtr, sizeof(struct ifaddr), 
																	&IfAddr);
		if (kmem_result == 0)
			{
			[MyMaster addMessage:"NetIfAddresses : error reading kernel.\n"];
			return self;
			}
			
		[theMatrix addRow];
		cellCount = [theMatrix cellCount];
		newCell = [theMatrix cellAt:(cellCount - 1) :0];
		switch (IfAddr.ifa_addr.sa_family)
			{
			case AF_UNSPEC :
				sprintf(tmpChar, "Unspecified %i", i++);
				break;
			case AF_UNIX :
				sprintf(tmpChar, "Unix %i", i++);
				break;
			case AF_INET :
				sprintf(tmpChar, "TCP/IP %i", i++);
				break;
			case AF_IMPLINK :
				sprintf(tmpChar, "IMP %i", i++);
				break;
			case AF_PUP :
				sprintf(tmpChar, "PUP %i", i++);
				break;
			case AF_CCITT :
				sprintf(tmpChar, "CCITT %i", i++);
				break;
			case AF_SNA :
				sprintf(tmpChar, "SNA %i", i++);
				break;
			case AF_DECnet :
				sprintf(tmpChar, "DECnet %i", i++);
				break;
			case AF_LAT :
				sprintf(tmpChar, "LAT %i", i++);
				break;
			case AF_APPLETALK :
				sprintf(tmpChar, "AppleTalk %i", i++);
				break;
			default :
				sprintf(tmpChar, "Unknown %i", i++);
				break;
			}
		[newCell setStringValue:tmpChar];
		[newCell setFreePointer:(void *)ifaddrPtr];		// This is not great
		[newCell setUpdater:self];
		[newCell setDir:[theCell getDir]];
		[newCell setLeaf:YES];
		ifaddrPtr = IfAddr.ifa_next;
		}
	while (ifaddrPtr);
	}
else
	{
	}
	

[theCell setMasterInstance:self];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetIfAddresses : error closing kmem_file.\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 			kmem_file;
struct ifaddr	IfAddr;
char			tmpChar[64], tmpChar2[64];
int				i;
unsigned int	tmpIntArray[14];

/////// Get list of interfaces
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetIfAddresses : error opening KMEM (2).\n"];
	return self;
	}

getkval(kmem_file, (int)[theCell freePointer], sizeof(struct ifaddr), &IfAddr);
for (i = 0; i < 18; i++)
	[[familyMatrix findCellWithTag:i] setState:0];
[[familyMatrix findCellWithTag:IfAddr.ifa_addr.sa_family] setState:1];

switch (IfAddr.ifa_addr.sa_family)
	{
	case AF_INET :
		for (i = 0; i < 4; i++)
			tmpIntArray[i] = (IfAddr.ifa_addr.sa_data[i + 2] & 0xff);
		sprintf(tmpChar, "%i.%i.%i.%i", tmpIntArray[0], 
						tmpIntArray[1], tmpIntArray[2], tmpIntArray[3]);
		for (i = 0; i < 4; i++)
			tmpIntArray[i] = (IfAddr.ifa_broadaddr.sa_data[i + 2] & 0xff);
		sprintf(tmpChar2, "%i.%i.%i.%i", tmpIntArray[0], 
						tmpIntArray[1], tmpIntArray[2], tmpIntArray[3]);
		break;
		
	default :
		for (i = 0; i < 14; i++)
			tmpIntArray[i] = (IfAddr.ifa_addr.sa_data[i] & 0xff);
		sprintf(tmpChar, "%2.2x.%2.2x.%2.2x.%2.2x.%2.2x.%2.2x...", 
			tmpIntArray[0], tmpIntArray[1], tmpIntArray[2], tmpIntArray[3], 
			tmpIntArray[4], tmpIntArray[5]);
		for (i = 0; i < 14; i++)
			tmpIntArray[i] = (IfAddr.ifa_broadaddr.sa_data[i] & 0xff);
		sprintf(tmpChar2, "%2.2x.%2.2x.%2.2x.%2.2x.%2.2x.%2.2x...", 
			tmpIntArray[0], tmpIntArray[1], tmpIntArray[2], tmpIntArray[3], 
			tmpIntArray[4], tmpIntArray[5]);
		break;
	}
[address setStringValue:tmpChar];
[broadAddress setStringValue:tmpChar2];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetIfAddresses : error closing kmem_file (2).\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
