// NetInterfaces.m
//
// History :
//		10.31.1992 : created by Max Tardiveau

#import "NetInterfaces.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation NetInterfaces


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetInterfaces : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetInterfaces : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int				cellCount;
id				newCell;
int 			kmem_file;
struct nlist	corelist[2];
struct ifnet	*ifnetPtr, IfNet;
char			tmpChar[16], tmpChar2[16];

if ([theCell updater])		// It's an interface
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

/////// Get list of interfaces
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "NetInterfaces : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_ifnet";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "NetInterfaces : error in nlist().\n");
	exit(1);
	}

getkval(kmem_file,corelist[0].n_value, sizeof(struct ifnet *), &ifnetPtr);
getkval(kmem_file, (int)ifnetPtr, sizeof(struct ifnet), &IfNet);

for (;;)		// This assumes there is at least one interface (!)
	{
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	getkval(kmem_file, (int)IfNet.if_name, 6, tmpChar);
	sprintf(tmpChar2, "%s%i", tmpChar, IfNet.if_unit);
	[newCell setStringValue:tmpChar2];
	[newCell setFreePointer:(void *)ifnetPtr];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:NO];

	if ( ! IfNet.if_next)
		break;
	ifnetPtr = IfNet.if_next;
	getkval(kmem_file, (int)ifnetPtr, sizeof(struct ifnet), &IfNet);
	}

[theCell setMasterInstance:self];

//if (close(kmem_file) < 0)
//	fprintf(stderr, "NetInterfaces : error closing kmem_file.\n");
close(kmem_file);

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 			kmem_file;
struct nlist	corelist[2];
struct ifnet	*ifnetPtr, IfNet;
char			tmpChar[64];
int				i, foundIt;

/////// Find the interface
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "NetInterfaces : error opening KMEM (2).\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_ifnet";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "NetInterfaces : error in nlist(2).\n");
	exit(1);
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct ifnet *), &ifnetPtr);
getkval(kmem_file, (int)ifnetPtr, sizeof(struct ifnet), &IfNet);
foundIt = NO;
for (;;)		// This assumes there is at least one interface (!)
	{
	getkval(kmem_file, (int)IfNet.if_name, 6, tmpChar);
	if ([theCell freePointer] == ifnetPtr)
		{
		foundIt = YES;
		break;
		}
	
	if ( ! IfNet.if_next)
		break;
	ifnetPtr = IfNet.if_next;
	getkval(kmem_file, (int)IfNet.if_next,
									sizeof(struct ifnet), &IfNet);
	}

if (foundIt)
	{
	[ifName setStringValue:tmpChar];
	getkval(kmem_file, (int)IfNet.if_type, 24, tmpChar);
	[ifType setStringValue:tmpChar];
	[subUnit setIntValue:IfNet.if_unit];
	[MTU setIntValue:IfNet.if_mtu];
	[routing setIntValue:IfNet.if_metric];
	
	[[flagMatrix findCellWithTag:0] setIntValue:(IfNet.if_flags & IFF_UP)];
	[[flagMatrix findCellWithTag:1]
								setIntValue:(IfNet.if_flags & IFF_BROADCAST)];
	[[flagMatrix findCellWithTag:2] setIntValue:(IfNet.if_flags & IFF_DEBUG)];
	[[flagMatrix findCellWithTag:3]
								setIntValue:(IfNet.if_flags & IFF_LOOPBACK)];
	[[flagMatrix findCellWithTag:4]
							setIntValue:(IfNet.if_flags & IFF_POINTOPOINT)];
	[[flagMatrix findCellWithTag:5]
							setIntValue:(IfNet.if_flags & IFF_NOTRAILERS)];
	[[flagMatrix findCellWithTag:6]
								setIntValue:(IfNet.if_flags & IFF_RUNNING)];
	[[flagMatrix findCellWithTag:7] setIntValue:(IfNet.if_flags & IFF_NOARP)];
	
	for (i = 0; i < 3; i++)
		[[ifClassMatrix cellAt:i :0] setState:0];
	if (IfNet.if_class == 0)
		[[ifClassMatrix cellAt:0 :0] setState:1];
	if (IfNet.if_class == 0x1000)		// Found in <net/netif.h>
		[[ifClassMatrix cellAt:1 :0] setState:1];
	if (IfNet.if_class == 0x2000)
		[[ifClassMatrix cellAt:2 :0] setState:1];
	[inPackets setIntValue:IfNet.if_ipackets];
	[inErrors setIntValue:IfNet.if_ierrors];
	[outPackets setIntValue:IfNet.if_opackets];
	[outErrors setIntValue:IfNet.if_oerrors];
	[collisions setIntValue:IfNet.if_collisions];
	}
else
	{
	[MyMaster addMessage:"NetInterfaces : could not find interface.\n"];
	[ifName setStringValue:"Error"];
	}

[theCell setMasterInstance:self];

if (close(kmem_file) < 0)
	fprintf(stderr, "NetInterfaces : error closing kmem_file(2).\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
