// NetICMPstats.m
//

#import "NetICMPstats.h"

@implementation NetICMPstats


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetICMPstats : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetICMPstats : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct	icmpstat		ICMPStat;

[theCell setUpdater:self];

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetICMPstats : error opening KMEM.\n"];;
	return self;
	}

corelist[0].n_un.n_name = "_icmpstat";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"NetICMPstats : error in nlist(2).\n"];
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct icmpstat), &ICMPStat);

[callsError setIntValue:ICMPStat.icps_error];
[oldICMP setIntValue:ICMPStat.icps_oldicmp];
[badCode setIntValue:ICMPStat.icps_badcode];
[minLength setIntValue:ICMPStat.icps_tooshort];
[badChecksum setIntValue:ICMPStat.icps_checksum];
[badLength setIntValue:ICMPStat.icps_badlen];
[responseGen setIntValue:ICMPStat.icps_reflect];

[[sendMatrix findCellWithTag:ICMP_ECHOREPLY]
					setIntValue:ICMPStat.icps_outhist[ICMP_ECHOREPLY]];
[[receiveMatrix findCellWithTag:ICMP_ECHOREPLY]
					setIntValue:ICMPStat.icps_inhist[ICMP_ECHOREPLY]];

[[sendMatrix findCellWithTag:ICMP_UNREACH]
					setIntValue:ICMPStat.icps_outhist[ICMP_UNREACH]];
[[receiveMatrix findCellWithTag:ICMP_UNREACH]
					setIntValue:ICMPStat.icps_inhist[ICMP_UNREACH]];

[[sendMatrix findCellWithTag:ICMP_SOURCEQUENCH]
					setIntValue:ICMPStat.icps_outhist[ICMP_SOURCEQUENCH]];
[[receiveMatrix findCellWithTag:ICMP_SOURCEQUENCH]
					setIntValue:ICMPStat.icps_inhist[ICMP_SOURCEQUENCH]];

[[sendMatrix findCellWithTag:ICMP_REDIRECT]
					setIntValue:ICMPStat.icps_outhist[ICMP_REDIRECT]];
[[receiveMatrix findCellWithTag:ICMP_REDIRECT]
					setIntValue:ICMPStat.icps_inhist[ICMP_REDIRECT]];

[[sendMatrix findCellWithTag:ICMP_ECHO]
					setIntValue:ICMPStat.icps_outhist[ICMP_ECHO]];
[[receiveMatrix findCellWithTag:ICMP_ECHO]
					setIntValue:ICMPStat.icps_inhist[ICMP_ECHO]];

[[sendMatrix findCellWithTag:ICMP_TIMXCEED]
					setIntValue:ICMPStat.icps_outhist[ICMP_TIMXCEED]];
[[receiveMatrix findCellWithTag:ICMP_TIMXCEED]
					setIntValue:ICMPStat.icps_inhist[ICMP_TIMXCEED]];

[[sendMatrix findCellWithTag:ICMP_PARAMPROB]
					setIntValue:ICMPStat.icps_outhist[ICMP_PARAMPROB]];
[[receiveMatrix findCellWithTag:ICMP_PARAMPROB]
					setIntValue:ICMPStat.icps_inhist[ICMP_PARAMPROB]];

[[sendMatrix findCellWithTag:ICMP_TSTAMP]
					setIntValue:ICMPStat.icps_outhist[ICMP_TSTAMP]];
[[receiveMatrix findCellWithTag:ICMP_TSTAMP]
					setIntValue:ICMPStat.icps_inhist[ICMP_TSTAMP]];

[[sendMatrix findCellWithTag:ICMP_TSTAMPREPLY]
					setIntValue:ICMPStat.icps_outhist[ICMP_TSTAMPREPLY]];
[[receiveMatrix findCellWithTag:ICMP_TSTAMPREPLY]
					setIntValue:ICMPStat.icps_inhist[ICMP_TSTAMPREPLY]];

[[sendMatrix findCellWithTag:ICMP_IREQ]
					setIntValue:ICMPStat.icps_outhist[ICMP_IREQ]];
[[receiveMatrix findCellWithTag:ICMP_IREQ]
					setIntValue:ICMPStat.icps_inhist[ICMP_IREQ]];

[[sendMatrix findCellWithTag:ICMP_IREQREPLY]
					setIntValue:ICMPStat.icps_outhist[ICMP_IREQREPLY]];
[[receiveMatrix findCellWithTag:ICMP_IREQREPLY]
					setIntValue:ICMPStat.icps_inhist[ICMP_IREQREPLY]];

[[sendMatrix findCellWithTag:ICMP_MASKREQ]
					setIntValue:ICMPStat.icps_outhist[ICMP_MASKREQ]];
[[receiveMatrix findCellWithTag:ICMP_MASKREQ]
					setIntValue:ICMPStat.icps_inhist[ICMP_MASKREQ]];

[[sendMatrix findCellWithTag:ICMP_MASKREPLY]
					setIntValue:ICMPStat.icps_outhist[ICMP_MASKREPLY]];
[[receiveMatrix findCellWithTag:ICMP_MASKREPLY]
					setIntValue:ICMPStat.icps_inhist[ICMP_MASKREPLY]];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetICMPstats : error closing kmem_file (2).\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
