// NetIPstats.m
//

#import "NetIPstats.h"

@implementation NetIPstats


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetIPstats : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetIPstats : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct	ipstat			IPStat;

[theCell setUpdater:self];

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetIPstats : error opening KMEM.\n"];;
	return self;
	}

corelist[0].n_un.n_name = "_ipstat";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"NetIPstats : error in nlist(2).\n"];
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct ipstat), &IPStat);

[totalPackets setIntValue:IPStat.ips_total];
[checksumBad setIntValue:IPStat.ips_badsum];
[tooShort setIntValue:IPStat.ips_tooshort];
[notEnoughData setIntValue:IPStat.ips_toosmall];
[headerLength setIntValue:IPStat.ips_badhlen];
[IPlength setIntValue:IPStat.ips_badlen];
[fragsReceived setIntValue:IPStat.ips_fragments];
[fragsDropDup setIntValue:IPStat.ips_fragdropped];
[fragsDropTime setIntValue:IPStat.ips_fragtimeout];
[forwarded setIntValue:IPStat.ips_forward];
[notForward setIntValue:IPStat.ips_cantforward];
[redirects setIntValue:IPStat.ips_redirectsent];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetIPstats : error closing kmem_file (2).\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
