// NetMBUFstats.m
//

#import "NetMBUFstats.h"

@implementation NetMBUFstats


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetMBUFstats : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetMBUFstats : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct	mbstat			MBUFStat;
int						i, pagesize, totalAlloc, inUse = 0;

[theCell setUpdater:self];

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetMBUFstats : error opening KMEM.\n"];;
	return self;
	}

corelist[0].n_un.n_name = "_mbstat";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"NetMBUFstats : error in nlist(2).\n"];
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct mbstat), &MBUFStat);

[mbufsInUse setIntValue:(MBUFStat.m_mbufs - MBUFStat.m_mtypes[MT_FREE])];
[mbufsTotal setIntValue:MBUFStat.m_mbufs];
for (i = 0; i < MT_IFADDR+1; i++)
	[[mbufTypesMatrix findCellWithTag:i] setIntValue:MBUFStat.m_mtypes[i]];
[clustersInUse setIntValue:MBUFStat.m_clusters - MBUFStat.m_clfree];
[clustersTotal setIntValue:MBUFStat.m_clusters];
[pagesAllocated setIntValue:MBUFStat.m_space];

pagesize = getpagesize();
totalAlloc = (MBUFStat.m_mbufs * MSIZE) + (MBUFStat.m_clusters * MCLBYTES) + 
												(MBUFStat.m_space * pagesize);
[netAlloc setIntValue:totalAlloc / 1024];
for (i = 0; i < MT_IFADDR+1; i++)
	inUse += (MBUFStat.m_mtypes[i] * MSIZE);
inUse += (MBUFStat.m_clusters - MBUFStat.m_clfree) * MCLBYTES;
[netAllocInUse setIntValue:((totalAlloc - inUse) * 100)/totalAlloc];
[requestsDenied setIntValue:MBUFStat.m_drops];
[requestsDelayed setIntValue:MBUFStat.m_wait];
[callsToDrain setIntValue:MBUFStat.m_drain];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetMBUFstats : error closing kmem_file (2).\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
