// NetTCPstats.m
//

#import "NetTCPstats.h"

@implementation NetTCPstats


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "NetTCPstats : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "NetTCPstats : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct	tcpstat			TCPStat;

[theCell setUpdater:self];

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"NetTCPstats : error opening KMEM.\n"];
	return self;
	}

corelist[0].n_un.n_name = "_tcpstat";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"NetTCPstats : error in nlist(2).\n"];
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct tcpstat), &TCPStat);

[connectionRequests setIntValue:TCPStat.tcps_connattempt];
[connectionAccepts setIntValue:TCPStat.tcps_accepts];
[connectionsEstablished setIntValue:TCPStat.tcps_connects];
[connectionsClosed setIntValue:TCPStat.tcps_closed];
[connectionDrops setIntValue:TCPStat.tcps_drops];
[embryoDrops setIntValue:TCPStat.tcps_conndrops];
[segmentsRTT setIntValue:TCPStat.tcps_rttupdated];
[segmentsRTTattempts setIntValue:TCPStat.tcps_segstimed];
[retransmitTimeouts setIntValue:TCPStat.tcps_rexmttimeo];
[dropsByRexmitTimeouts setIntValue:TCPStat.tcps_timeoutdrop];
[persistTimeouts setIntValue:TCPStat.tcps_persisttimeo];
[keepaliveTimeouts setIntValue:TCPStat.tcps_keeptimeo];
[keepaliveProbes setIntValue:TCPStat.tcps_keepprobe];
[dropsByKeepalive setIntValue:TCPStat.tcps_keepdrops];
// Sending stats
[totalSent setIntValue:TCPStat.tcps_sndtotal];
[sentDataPackets setIntValue:TCPStat.tcps_sndpack];
[sentDataBytes setIntValue:TCPStat.tcps_sndbyte];
[sentRexmitData setIntValue:TCPStat.tcps_sndrexmitpack];
[sentRexmitBytes setIntValue:TCPStat.tcps_sndrexmitbyte];
[sentACKonly setIntValue:TCPStat.tcps_sndacks];
[sentACKdelayed setIntValue:TCPStat.tcps_delack];
[sentURGonly setIntValue:TCPStat.tcps_sndurg];
[sentWindowProbe setIntValue:TCPStat.tcps_sndprobe];
[sentWindowUpdate setIntValue:TCPStat.tcps_sndwinup];
[sentControl setIntValue:TCPStat.tcps_sndctrl];
// Received stats
[totalReceived setIntValue:TCPStat.tcps_rcvtotal];
[recACKs setIntValue:TCPStat.tcps_rcvackpack];
[recACKbytes setIntValue:TCPStat.tcps_rcvackbyte];
[recDupACKS setIntValue:TCPStat.tcps_rcvdupack];
[recACKforUnsent setIntValue:TCPStat.tcps_rcvacktoomuch];
[recInSequence setIntValue:TCPStat.tcps_rcvpack];
[recInSequenceBytes setIntValue:TCPStat.tcps_rcvbyte];
[recCompDup setIntValue:TCPStat.tcps_rcvduppack];
[recCompDupBytes setIntValue:TCPStat.tcps_rcvdupbyte];
[recPartDup setIntValue:TCPStat.tcps_rcvpartduppack];
[recPartDupBytes setIntValue:TCPStat.tcps_rcvpartdupbyte];
[recOOO setIntValue:TCPStat.tcps_rcvoopack];
[recOOObytes setIntValue:TCPStat.tcps_rcvoobyte];
[recAfterWindow setIntValue:TCPStat.tcps_rcvpackafterwin];
[recAfterWindowBytes setIntValue:TCPStat.tcps_rcvbyteafterwin];
[recWinProbes setIntValue:TCPStat.tcps_rcvwinprobe];
[recWinUpdates setIntValue:TCPStat.tcps_rcvwinupd];
[recAfterClose setIntValue:TCPStat.tcps_rcvafterclose];
[recBadChecksum setIntValue:TCPStat.tcps_rcvbadsum];
[recBadHeader setIntValue:TCPStat.tcps_rcvbadoff];
[recTooShort setIntValue:TCPStat.tcps_rcvshort];

if (close(kmem_file) < 0)
	[MyMaster addMessage:"NetTCPstats : error closing kmem_file (2).\n"];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 20;
}

@end
