// ProcessChildrenUsage.m
//
// History :
//		10.14.92 : created by Max Tardiveau

#import "ProcessChildrenUsage.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation ProcessChildrenUsage


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "Tasks : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "Tasks : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct proc				*proc;
struct proc				tmpProcess;
struct task				tmptask;
struct utask			tmputask;
unsigned int			tmpInt;

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Tasks : error opening KMEM.\n");
	return self;
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Tasks : error in nlist().\n");
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);             

tmpInt = NO;
for (;;)
	{
	if (tmpProcess.p_pid == (int)[theCell freePointer])
		{
		tmpInt = YES;
		break;
		}
		
	if ( ! tmpProcess.p_nxt)
		break;
	
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

if (tmpInt)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	[userTimeSec setIntValue:tmputask.uu_ru.ru_utime.tv_sec];
	[userTimeUSec setIntValue:tmputask.uu_ru.ru_utime.tv_usec];
	[systemTimeSec setIntValue:tmputask.uu_ru.ru_stime.tv_sec];
	[systemTimeUSec setIntValue:tmputask.uu_ru.ru_stime.tv_usec];
	[contextVol setIntValue:tmputask.uu_ru.ru_nvcsw];
	[contextUnvol setIntValue:tmputask.uu_ru.ru_nivcsw];
	[blockInput setIntValue:tmputask.uu_ru.ru_inblock];
	[blockOutput setIntValue:tmputask.uu_ru.ru_oublock];
	[messagesSent setIntValue:tmputask.uu_ru.ru_msgsnd];
	[messagesRec setIntValue:tmputask.uu_ru.ru_msgrcv];
	[signalsRec setIntValue:tmputask.uu_ru.ru_nsignals];
	[faultsMinor setIntValue:tmputask.uu_ru.ru_minflt];
	[faultsMajor setIntValue:tmputask.uu_ru.ru_majflt];
	[swaps setIntValue:tmputask.uu_ru.ru_nswap];
	}
else
	{
	[userTimeSec setStringValue:"Error"];
	[userTimeUSec setStringValue:"Error"];
	[systemTimeSec setStringValue:"Error"];
	[systemTimeUSec setStringValue:"Error"];
	[contextVol setStringValue:"Error"];
	[contextUnvol setStringValue:"Error"];
	[blockInput setStringValue:"Error"];
	[blockOutput setStringValue:"Error"];
	[messagesSent setStringValue:"Error"];
	[messagesRec setStringValue:"Error"];
	[signalsRec setStringValue:"Error"];
	[faultsMinor setStringValue:"Error"];
	[faultsMajor setStringValue:"Error"];
	[swaps setStringValue:"Error"];
	}

// For some reason, the close always fails. I ignore the error for now.
//if (close(kmem_file) < 0)
//	fprintf(stderr, "ProcessLimits : error closing kmem_file (2).\n");
close(kmem_file);

return self;
}

@end
