// Processes.m
//
// History :
//		10.12.1992 : created by Max Tardiveau

#import "Processes.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation Processes


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "Processes : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "Processes : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int				cellCount;
id				newCell;
int 			kmem_file;
struct nlist	corelist[2];
struct proc		*proc;
struct proc		tmpProcess;
struct task		tmptask;
struct utask	tmputask;

if ([theCell updater])		// It's a process
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

/////// Get list of processes
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Processes : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Processes : error in nlist().\n");
	exit(1);
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);

for (;;)		// This assumes there is at least one process (!)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	[newCell setStringValue:(char *)&(tmputask.uu_comm)];
	[newCell setFreePointer:(void *)(0L + tmpProcess.p_pid)];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:NO];

	if ( ! tmpProcess.p_nxt)
		break;
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

[theCell setMasterInstance:self];

// For some reason, the close always fails. I ignore the error for now.
//if (close(kmem_file) < 0)
//	fprintf(stderr, "Processes : error closing kmem_file.\n");
close(kmem_file);

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct proc				*proc;
struct proc				tmpProcess;
struct task				tmptask;
struct utask			tmputask;
unsigned int			tmpInt;
struct passwd			*thePassword;
int						i, j;

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Processes : error opening KMEM.\n");
	return self;
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Processes : error in nlist().\n");
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);             

tmpInt = NO;
for (;;)
	{
	if (tmpProcess.p_pid == (int)[theCell freePointer])
		{
		tmpInt = YES;
		break;
		}
		
	if ( ! tmpProcess.p_nxt)
		break;
	
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

if (tmpInt)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	[processName setStringValue:(char *)&(tmputask.uu_comm)];
	[processID setIntValue:tmpProcess.p_pid];
	
	thePassword = getpwuid(tmpProcess.p_uid);
	
	[userName setStringValue:thePassword->pw_name];
	[groupLeader setIntValue:tmpProcess.p_pgrp];
	[parentID setIntValue:tmpProcess.p_ppid];
	[userPriority setIntValue:tmpProcess.p_usrpri];
	[realPriority setIntValue:tmpProcess.p_pri];
	[niceValue setIntValue:tmpProcess.p_nice];
	[[fileMaskMatrix cellAt:0 :0] setState:!(tmputask.uu_cmask & 00400)];
	[[fileMaskMatrix cellAt:1 :0] setState:!(tmputask.uu_cmask & 00200)];
	[[fileMaskMatrix cellAt:2 :0] setState:!(tmputask.uu_cmask & 00100)];
	[[fileMaskMatrix cellAt:0 :1] setState:!(tmputask.uu_cmask & 00040)];
	[[fileMaskMatrix cellAt:1 :1] setState:!(tmputask.uu_cmask & 00020)];
	[[fileMaskMatrix cellAt:2 :1] setState:!(tmputask.uu_cmask & 00010)];
	[[fileMaskMatrix cellAt:0 :2] setState:!(tmputask.uu_cmask & 00004)];
	[[fileMaskMatrix cellAt:1 :2] setState:!(tmputask.uu_cmask & 00002)];
	[[fileMaskMatrix cellAt:2 :2] setState:!(tmputask.uu_cmask & 00001)];
	}
else
	{
	[processName setStringValue:"Not found"];
	[processID setStringValue:"Error"];
	[userName setStringValue:"Error"];
	[groupLeader setStringValue:"Error"];
	[parentID setStringValue:"Error"];
	[userPriority setStringValue:"Error"];
	[realPriority setStringValue:"Error"];
	[niceValue setStringValue:"Error"];
	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			[[fileMaskMatrix cellAt:j :i] setState:0];
	}

// For some reason, the close always fails. I ignore the error for now.
//if (close(kmem_file) < 0)
//	fprintf(stderr, "Processes : error closing kmem_file (2).\n");
close(kmem_file);

return self;
}

@end
