// ProcessLimits.m
//
// History :
//		10.14.92 : created by Max Tardiveau

#import "ProcessLimits.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation ProcessLimits


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "Tasks : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "Tasks : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doInfinityFor:theCell value:(int) theValue
{
if (theValue == RLIM_INFINITY)
	[theCell setStringValue:"No limit"];
else
	[theCell setIntValue:theValue];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct proc				*proc;
struct proc				tmpProcess;
struct task				tmptask;
struct utask			tmputask;
unsigned int			tmpInt;

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Tasks : error opening KMEM.\n");
	return self;
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Tasks : error in nlist().\n");
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);             

tmpInt = NO;
for (;;)
	{
	if (tmpProcess.p_pid == (int)[theCell freePointer])
		{
		tmpInt = YES;
		break;
		}
		
	if ( ! tmpProcess.p_nxt)
		break;
	
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

if (tmpInt)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	[self doInfinityFor:CPUtimeCurrent
						value:tmputask.uu_rlimit[RLIMIT_CPU].rlim_cur];
	[self doInfinityFor:CPUtimeMax
						value:tmputask.uu_rlimit[RLIMIT_CPU].rlim_max];
	[self doInfinityFor:fileSizeCurrent
						value:tmputask.uu_rlimit[RLIMIT_FSIZE].rlim_cur];
	[self doInfinityFor:fileSizeMax
						value:tmputask.uu_rlimit[RLIMIT_FSIZE].rlim_max];
	[self doInfinityFor:dataSizeCurrent
						value:tmputask.uu_rlimit[RLIMIT_DATA].rlim_cur];
	[self doInfinityFor:dataSizeMax
						value:tmputask.uu_rlimit[RLIMIT_DATA].rlim_max];
	[self doInfinityFor:stackSizeCurrent
						value:tmputask.uu_rlimit[RLIMIT_STACK].rlim_cur];
	[self doInfinityFor:stackSizeMax
						value:tmputask.uu_rlimit[RLIMIT_STACK].rlim_max];
	[self doInfinityFor:coreFileCurrent
						value:tmputask.uu_rlimit[RLIMIT_CORE].rlim_cur];
	[self doInfinityFor:coreFileMax
						value:tmputask.uu_rlimit[RLIMIT_CORE].rlim_max];
	[self doInfinityFor:residentSetCurrent
						value:tmputask.uu_rlimit[RLIMIT_RSS].rlim_cur];
	[self doInfinityFor:residentSetMax
						value:tmputask.uu_rlimit[RLIMIT_RSS].rlim_max];
	}
else
	{
	[CPUtimeCurrent setStringValue:"Error"];
	[CPUtimeMax setStringValue:"Error"];
	[fileSizeCurrent setStringValue:"Error"];
	[fileSizeMax setStringValue:"Error"];
	[dataSizeCurrent setStringValue:"Error"];
	[dataSizeMax setStringValue:"Error"];
	[stackSizeCurrent setStringValue:"Error"];
	[stackSizeMax setStringValue:"Error"];
	[coreFileCurrent setStringValue:"Error"];
	[coreFileMax setStringValue:"Error"];
	[residentSetCurrent setStringValue:"Error"];
	[residentSetMax setStringValue:"Error"];
	}

// For some reason, the close always fails. I ignore the error for now.
//if (close(kmem_file) < 0)
//	fprintf(stderr, "ProcessLimits : error closing kmem_file (2).\n");
close(kmem_file);

return self;
}

@end
