// ProcessSignals.m
//
// History :
//		10.14.92 : created by Max Tardiveau

#import "ProcessSignals.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation ProcessSignals


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "ProcessSignals : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "ProcessSignals : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 					kmem_file;
struct nlist			corelist[2];
struct proc				*proc;
struct proc				tmpProcess;
struct task				tmptask;
struct utask			tmputask;
unsigned int			tmpInt;
int						i;

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "ProcessSignals : error opening KMEM.\n");
	return self;
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "ProcessSignals : error in nlist().\n");
	close(kmem_file);
	return self;
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);             

tmpInt = NO;
for (;;)
	{
	if (tmpProcess.p_pid == (int)[theCell freePointer])
		{
		tmpInt = YES;
		break;
		}
		
	if ( ! tmpProcess.p_nxt)
		break;
	
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

if (tmpInt)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	for (i = 1; i < 32; i++)
		if ((int)tmputask.uu_signal[i] == 1)
			[[signalMatrix cellAt:(i-1)%8 :((i-1)/8)] setEnabled:NO];
		else
			[[signalMatrix cellAt:(i-1)%8 :((i-1)/8)]
								setIntValue:(int)tmputask.uu_signal[i]];
	}
else
	{
	fprintf(stderr, "ProcessSignals : could not find process.\n");
	for (i = 1; i < 32; i++)
				[[signalMatrix cellAt:(i-1)%8 :((i-1)/8)] setIntValue:0];
	}

// For some reason, the close always fails. I ignore the error for now.
//if (close(kmem_file) < 0)
//	fprintf(stderr, "ProcessSignals : error closing kmem_file (2).\n");
close(kmem_file);

return self;
}

@end
