// ProcessorGroups.m
//
// History :
//		10.02.1992 : created by Max Tardiveau

#import "ProcessorGroups.h"

@implementation ProcessorGroups


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
kern_return_t		error;
processor_array_t	list;
unsigned int		count;
int					i, j, cellCount;
processor_set_t		theSet;
char				tmpChar[512];
id					newCell;

if ([theCell updater])		// It's a processor set
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

error = host_processors(host_priv_self(), &list, &count);
if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"ProcessorGroups : Error calling host_processors. You must be root to use this.\n"];
	return self;
	}

for (i = 0; i < count; i++)
	{
	error = processor_get_assignment(list[i], &theSet);
	if (error != KERN_SUCCESS)
		{
		fprintf(stderr,
			"ProcessorGroups : Error calling processor_get_assignment.\n");
		exit(1);
		}
	
	cellCount = [theMatrix cellCount];
	for (j = 0; j < cellCount; j++)  // Check we don't already have it
		if ([[theMatrix cellAt:j :0] freePointer] == (void *)theSet)
			break;
	
	if (j >= cellCount)		// Ain't there : add it to the list
		{
		[theMatrix addRow];
		cellCount = [theMatrix cellCount];
		newCell = [theMatrix cellAt:(cellCount - 1) :0];
		sprintf(tmpChar, "Processor set %i", cellCount);
		[newCell setStringValue:tmpChar];
		[newCell setFreePointer:(void *)theSet];
		[newCell setUpdater:self];
		[newCell setDir:[theCell getDir]];
		}
	}

[theCell setMasterInstance:self];

error = vm_deallocate(task_self(), (vm_address_t)list, sizeof(list)*count);
if (error != KERN_SUCCESS)
	fprintf(stderr, "ProcessorGroups : trouble freeing list.\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t		error;
unsigned int		info_count;
struct processor_set_basic_info		basicInfo;
struct processor_set_sched_info		schedInfo;
host_t				host;

info_count = PROCESSOR_SET_BASIC_INFO_COUNT;

error = processor_set_info((processor_set_t)[theCell freePointer], 
				PROCESSOR_SET_BASIC_INFO, &host,
				(processor_set_info_t)&basicInfo, &info_count);
				
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "ProcessorGroups : processor_set_info.\n");
	[numProcessors setStringValue:"Error"];
	[numTasks setStringValue:"Error"];
	[numThreads setStringValue:"Error"];
	[loadAverage setStringValue:"Error"];
	[machFactor setStringValue:"Error"];
	}
else
	{
	[numProcessors setIntValue:basicInfo.processor_count];
	[numTasks setIntValue:basicInfo.task_count];
	[numThreads setIntValue:basicInfo.thread_count];
	[loadAverage setFloatValue:((float)basicInfo.load_average/LOAD_SCALE)];
	[machFactor setFloatValue:((float)basicInfo.mach_factor/LOAD_SCALE)];
	}

info_count = PROCESSOR_SET_SCHED_INFO_COUNT;

error = processor_set_info((processor_set_t)[theCell freePointer], 
				PROCESSOR_SET_SCHED_INFO, &host,
				(processor_set_info_t)&schedInfo, &info_count);

if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "ProcessorGroups : processor_set_info.\n");
	[maxPriority setStringValue:"Error"];
	[timeSharing setState:0];
	[interactive setState:0];
	[fixedPriority setState:0];
	}
else
	{
	[maxPriority setIntValue:schedInfo.max_priority];
	[timeSharing setState:schedInfo.policies & POLICY_TIMESHARE];
	[interactive setState:schedInfo.policies & POLICY_INTERACTIVE];
	[fixedPriority setState:schedInfo.policies & POLICY_FIXEDPRI];
	}

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 5;
}

@end
