// Processors.m
//
// History :
//		10.03.1992 : created by Max Tardiveau

#import "Processors.h"

@implementation Processors


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
kern_return_t		error;
processor_array_t	list;
processor_set_t		theSet;
unsigned int		count;
int					i, cellCount;
id					newCell;
char				tmpChar[256];

error = host_processors(host_priv_self(), &list, &count);
if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"Processors : Error calling host_processors. You must be root to use this.\n"];
	return self;
	}

for (i = 0; i < count; i++)
	{
	error = processor_get_assignment(list[i], &theSet);
	if (error != KERN_SUCCESS)
		{
		fprintf(stderr,
				"Processors : Error calling processor_get_assignment.\n");
		exit(1);
		}

	if ([theCell freePointer] == (void *)theSet)	// Belongs to our set
		{
		[theMatrix addRow];
		cellCount = [theMatrix cellCount];
		newCell = [theMatrix cellAt:(cellCount - 1) :0];
		sprintf(tmpChar, "Processor %i", cellCount);
		[newCell setStringValue:tmpChar];
		[newCell setFreePointer:(void *)list[i]];
		[newCell setUpdater:self];
		[newCell setDir:[theCell getDir]];
		[newCell setLeaf:YES];
		}
	}

[theCell setMasterInstance:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t					error;
host_t							host;
unsigned int					info_count;
struct processor_basic_info		basic_info;
char							*type, *subtype;

info_count = PROCESSOR_BASIC_INFO_COUNT;
error = processor_info((port_t) [theCell freePointer], PROCESSOR_BASIC_INFO, 
		&host, (processor_info_t)&basic_info, &info_count);
if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"Processors : Error calling processor_info. You must be root to use this.\n"];
	[CPUtype setStringValue:"Error"];
	[CPUsubtype setStringValue:"Error"];
	[slotNumber setStringValue:"Error"];
	[isRunning setState:0];
	[isMaster setState:0];
	return self;
	}

switch (basic_info.cpu_type)
	{
	case CPU_TYPE_VAX :
		type = "Vax";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_VAX780 :	subtype = "Vax 780"; break;
			case CPU_SUBTYPE_VAX785 :	subtype = "Vax 785"; break;
			case CPU_SUBTYPE_VAX750 :	subtype = "Vax 750"; break;
			case CPU_SUBTYPE_VAX730 :	subtype = "Vax 730"; break;
			case CPU_SUBTYPE_UVAXI :	subtype = "MicroVax I"; break;
			case CPU_SUBTYPE_UVAXII :	subtype = "MicroVax II"; break;
			case CPU_SUBTYPE_VAX8200 :	subtype = "Vax 8200"; break;
			case CPU_SUBTYPE_VAX8500 :	subtype = "Vax 8500"; break;
			case CPU_SUBTYPE_VAX8600 :	subtype = "Vax 8600"; break;
			case CPU_SUBTYPE_VAX8650 :	subtype = "Vax 8650"; break;
			case CPU_SUBTYPE_VAX8800 :	subtype = "Vax 8800"; break;
			case CPU_SUBTYPE_UVAXIII :	subtype = "MicroVax III"; break;
			default :					subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_ROMP :
		type = "Romp";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_RT_PC :	subtype = "RT PC"; break;
			case CPU_SUBTYPE_RT_APC :	subtype = "RT APC"; break;
			case CPU_SUBTYPE_RT_135 :	subtype = "RT 135"; break;
			default :					subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_NS32032 :
	case CPU_TYPE_NS32332 :
	case CPU_TYPE_NS32532 :
		switch (basic_info.cpu_type)
			{
			case CPU_TYPE_NS32032 :	type = "NS 32032";		break;
			case CPU_TYPE_NS32332 : type = "NS 32332"; 		break;
			case CPU_TYPE_NS32532 : type = "NS 32532"; 		break;
			default :				type = "Unknown";		break;
			}
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_MMAX_DPC :	subtype = "032 CPU"; break;
			case CPU_SUBTYPE_SQT :		subtype = "SQT"; break;
			case CPU_SUBTYPE_MMAX_APC_FPU :	subtype = "32081 FPU"; break;
			case CPU_SUBTYPE_MMAX_APC_FPA :	subtype = "Weitek FPA"; break;
			case CPU_SUBTYPE_MMAX_XPC :	subtype = "532 CPU"; break;
			default :					subtype = "Unknown"; break;
			}
		break;

	case CPU_TYPE_I386 :
		type = "Intel 80x86";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_386 :		subtype = "386"; break;
			case CPU_SUBTYPE_486 :		subtype = "486"; break;
			case CPU_SUBTYPE_486SX :	subtype = "486SX"; break;
			case CPU_SUBTYPE_586 :		subtype = "Pentium"; break;
			case CPU_SUBTYPE_586SX :	subtype = "Pentium SX"; break;
			default :					subtype = "Unknown"; break;
			}
		break;

	case CPU_TYPE_MIPS :
		type = "Mips";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_MIPS_R2300 :	subtype = "R2300"; break;
			case CPU_SUBTYPE_MIPS_R2600 :	subtype = "R2600"; break;
			case CPU_SUBTYPE_MIPS_R2800 :	subtype = "R2800"; break;
			case CPU_SUBTYPE_MIPS_R2000a :	subtype = "R2000a"; break;
			default :						subtype = "Unknown"; break;
			}
		break;

	case CPU_TYPE_MC680x0 :
		type = "Motorola 680x0";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_MC68030 :	subtype = "MC 68030"; break;
			case CPU_SUBTYPE_MC68040 :	subtype = "MC 68040"; break;
			default :					subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_HPPA :
		type = "Hewlett-Packard PA";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_HPPA_ALL : subtype = "Generic HP-PA"; break;
			case CPU_SUBTYPE_HPPA_825 :	subtype = "HP-PA 825"; break;
			case CPU_SUBTYPE_HPPA_835 :	subtype = "HP-PA 835"; break;
			case CPU_SUBTYPE_HPPA_840 :	subtype = "HP-PA 840"; break;
			case CPU_SUBTYPE_HPPA_850 :	subtype = "HP-PA 850"; break;
			case CPU_SUBTYPE_HPPA_855 :	subtype = "HP-PA 855"; break;
			default :					subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_ARM :
		type = "Acorn Risc Machine";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_ARM_A500_ARCH :	subtype = "A500 Arch"; break;
			case CPU_SUBTYPE_ARM_A500 :			subtype = "A500"; break;
			case CPU_SUBTYPE_ARM_A440 :			subtype = "A440"; break;
			case CPU_SUBTYPE_ARM_M4 :			subtype = "M4"; break;
			case CPU_SUBTYPE_ARM_A680 :			subtype = "A680"; break;
			default :							subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_MC88000 :
		type = "Motorola 88000";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_MC88000_ALL : 	subtype = "Generic"; break;
			case CPU_SUBTYPE_MC88100 :		subtype = "88100 or JPC"; break;
			case CPU_SUBTYPE_MC88110 :		subtype = "88110"; break;
			default :						subtype = "Unknown"; break;
			}
		break;
	
	case CPU_TYPE_MC98000 :
		type = "PowerPC 98000";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_MC98000_ALL :	subtype = "Generic"; break;
			case CPU_SUBTYPE_MC98601 :		subtype = "98601"; break;
			default :						subtype = "Unknown"; break;
			}
		break;
	
	case CPU_TYPE_I860 :
		type = "Intel i860 big-endian";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_I860_ALL :		subtype = "Generic"; break;
			case CPU_SUBTYPE_I860_860 :		subtype = "860"; break;
			default :						subtype = "Unknown"; break;
			}
		break;
	
	case CPU_TYPE_I860_LITTLE :
		type = "Intel i860 little-endian";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_I860_LITTLE_ALL :subtype = "Generic"; break;
			case CPU_SUBTYPE_I860_LITTLE :	 subtype = "860"; break;
			default :						subtype = "Unknown"; break;
			}
		break;
	
	case CPU_TYPE_RS6000 :
		type = "RS6000";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_RS6000_ALL :	subtype = "Generic"; break;
			case CPU_SUBTYPE_RS6000 :		subtype = "RS6000"; break;
			default :						subtype = "Unknown"; break;
			}
		break;
		
	case CPU_TYPE_SPARC :
		type = "Sparc";
		switch (basic_info.cpu_subtype)
			{
			case CPU_SUBTYPE_SUN4_ALL :	subtype = "Generic"; break;
			case CPU_SUBTYPE_SUN4_260 :	subtype = "Sun 4-260"; break;
			case CPU_SUBTYPE_SUN4_110 :	subtype = "Sun 4-110"; break;
			default :					subtype = "Unknown"; break;
			}
		break;
		
	default :
		type = "Unknown";
		subtype = "Unknown";
		break;
	}
[CPUtype setStringValue:type];
[CPUsubtype setStringValue:subtype];

[slotNumber setIntValue:basic_info.slot_num];
[isRunning setState:basic_info.running];
[isMaster setState:basic_info.is_master];

return self;
}

@end
