// TaskPorts.m
//
// History :
//		10.24.1992 : created by Max Tardiveau

#import "TaskPorts.h"


@implementation TaskPorts


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
int					i, cellCount;
id					newCell;
kern_return_t		result;
port_name_array_t	portNames;
unsigned int		port_names_count;
port_type_array_t	port_types;
char				tmpChar[32];

if ([theCell secondaryPointer] == 0)
	{
	result = port_names((task_t)[theCell freePointer], &portNames,  
							&port_names_count, &port_types, &port_names_count);
	if (result != KERN_SUCCESS)
		{
		[MyMaster addMessage:"TaskPorts : error in port_names.\n"];
		return self;
		}
	}
else		// It's a port set
	{
	result = port_set_status((task_t)[theCell freePointer],
					(port_set_name_t)[theCell secondaryPointer],  
					&portNames, &port_names_count);
	if (result != KERN_SUCCESS)
		{
		[MyMaster addMessage:"TaskPorts : error in port_set_status.\n"];
		return self;
		}
	}

for (i = 0; i < port_names_count; i++)
	{
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	sprintf(tmpChar, "Port %i", portNames[i]);
	[newCell setStringValue:tmpChar];
	[newCell setFreePointer:[theCell freePointer]];
	[newCell setSecondaryPointer:(void *)portNames[i]];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:YES];
	[newCell setEnabled:!(port_types[i] == PORT_TYPE_SET)];
	}

[theCell setMasterInstance:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t		result;
port_set_name_t		port_set_name;
int					num_msgs, backlog;
boolean_t			isOwner;

if ( ! [theCell isLeaf])
	{
	[portName setStringValue:""];
	[portSet setStringValue:""];
	[ownsPort setState:0];
	[numMessages setStringValue:""];
	[maxMessages setStringValue:""];
	return self;
	}

result = port_status((task_t)[theCell freePointer],
				(port_name_t) [theCell secondaryPointer], &port_set_name,
				&num_msgs, &backlog, &isOwner, &isOwner);
if (result != KERN_SUCCESS)
	{
	[MyMaster addMessage:"TaskPorts : error in port_status.\n"];
	[portName setStringValue:"Error"];
	[portSet setStringValue:"Error"];
	[ownsPort setState:0];
	[numMessages setStringValue:"Error"];
	[maxMessages setStringValue:"Error"];
    return self;
	}

[portName setIntValue:(int)[theCell secondaryPointer]];
if (port_set_name == PORT_NULL)
	[portSet setStringValue:""];
else
	[portSet setIntValue:(int)port_set_name];

[ownsPort setState:isOwner];
[numMessages setIntValue:(num_msgs >= 0 ? num_msgs : -num_msgs)];
[maxMessages setIntValue:backlog];

return self;
}


@end
