// Tasks.m
//
// History :
//		10.04.1992 : created by Max Tardiveau

#import "Tasks.h"


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

@implementation Tasks


////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "Tasks : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "Tasks : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
task_array_t	task_list;
unsigned int	task_count;
processor_set_t	set_priv;
kern_return_t	error;
int				i, j, cellCount;
id				newCell;
char			tmpChar[256];
int 			kmem_file;
struct nlist	corelist[2];
struct proc		*proc;
struct proc		tmpProcess;
struct task		tmptask;
struct utask	tmputask;
int				tmpInt;
task_t			tmpTaskPtr;


if ([theCell updater])		// It's a task
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

/////// Get list of tasks
error = host_processor_set_priv(host_priv_self(),
					(processor_set_t)[theCell freePointer], &set_priv);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Tasks : call to host_processor_set_priv() failed.\n");
    return self;
	}

error = processor_set_tasks(set_priv, &task_list, &task_count);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Tasks : call to processor_set_tasks() failed.\n");
    return self;
	}

/////// Get list of processes
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Tasks : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Tasks : error in nlist().\n");
	exit(1);
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);

for (;;)		// This assumes there is at least one process (!)
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	
	tmpInt = NO;
	for (i = 0; i < task_count; i++)
		{
		error = task_by_unix_pid(task_self(), tmpProcess.p_pid, &tmpTaskPtr);
		if (error != KERN_SUCCESS)
			{
			fprintf(stderr, "Tasks : call to task_by_unix_pid() failed.\n");
			return self;
			}
		
		if (tmpTaskPtr == task_list[i])
			{
			tmpInt = YES;
			task_list[i] = 0;
			break;
			}
		}
	
	if (tmpInt)
		{
		[theMatrix addRow];
		cellCount = [theMatrix cellCount];
		newCell = [theMatrix cellAt:(cellCount - 1) :0];
		[newCell setStringValue:(char *)&(tmputask.uu_comm)];
		[newCell setFreePointer:(void *)tmpTaskPtr];
		[newCell setUpdater:self];
		[newCell setDir:[theCell getDir]];
		[newCell setLeaf:NO];
		}
	
	if ( ! tmpProcess.p_nxt)
		break;
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

/////// Add the tasks that are not Unix processes
j = 1;
for (i = 0; i < task_count; i++)
	if (task_list[i])
		{
		[theMatrix addRow];
		cellCount = [theMatrix cellCount];
		newCell = [theMatrix cellAt:(cellCount - 1) :0];
		sprintf(tmpChar, "Mach task %i", j++);
		[newCell setStringValue:tmpChar];
		[newCell setFreePointer:(void *)task_list[i]];
		[newCell setUpdater:self];
		[newCell setDir:[theCell getDir]];
		[newCell setLeaf:NO];
		}

[theCell setMasterInstance:self];

error = vm_deallocate(task_self(), (vm_address_t) task_list, 
												sizeof(task_list)*task_count);
if (error != KERN_SUCCESS)
    fprintf(stderr, "Tasks : trouble freeing task_list.\n");

if (close(kmem_file) < 0)
	fprintf(stderr, "Tasks : error closing kmem_file.\n");

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t			error;
struct task_basic_info	info;
unsigned int			info_count=TASK_BASIC_INFO_COUNT;
char					tmpChar[40];
int 					kmem_file;
struct nlist			corelist[2];
struct proc				*proc;
struct proc				tmpProcess;
struct task				tmptask;
struct utask			tmputask;
unsigned int			tmpInt;
task_t					tmpTaskPtr;
thread_array_t			theThreadArray;

/////// Get basic info
error = task_info((task_t) [theCell freePointer], TASK_BASIC_INFO, 
										(task_info_t)&info, &info_count);

if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Tasks : error calling task_info().\n");
	[suspendCount setStringValue:"Error"];
	[basePriority setStringValue:"Error"];
	[virtualPages setStringValue:"Error"];
	[residentPages setStringValue:"Error"];
	[userRunTimeSec setStringValue:"Error"];
	[userRunTimeUSec setStringValue:"Error"];
	[systemRunTimeSec setStringValue:"Error"];
	[systemRunTimeUSec setStringValue:"Error"];
	[unixProcess setStringValue:"Error"];
	[unixPID setStringValue:"Error"];
	[numThreads setStringValue:"Error"];
	return self;
	}
else
	{
	[suspendCount setIntValue:info.suspend_count];
	[basePriority setIntValue:info.base_priority];

	sprintf(tmpChar, "%.3f", info.virtual_size/(1024.0 * 1024.0));
	[virtualPages setStringValue:tmpChar];

	sprintf(tmpChar, "%.3f", info.resident_size/(1024.0 * 1024.0));
	[residentPages setStringValue:tmpChar];
	
	[userRunTimeSec setIntValue:info.user_time.seconds];
	[userRunTimeUSec setIntValue:info.user_time.microseconds];
	[systemRunTimeSec setIntValue:info.system_time.seconds];
	[systemRunTimeUSec setIntValue:info.system_time.microseconds];
	}

/////// Get process information, if task is a process
if ((kmem_file = open(KMEM, 0)) < 0)
	{
	fprintf(stderr, "Tasks : error opening KMEM.\n");
	exit(1);
	}

corelist[0].n_un.n_name = "_allproc";
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	fprintf(stderr, "Tasks : error in nlist().\n");
	exit(1);
	}

getkval(kmem_file, corelist[0].n_value, sizeof(struct proc *), &proc);
getkval(kmem_file, (int)proc, sizeof(struct proc), &tmpProcess);             

tmpInt = NO;
for (;;)
	{
	error = task_by_unix_pid(task_self(), tmpProcess.p_pid, &tmpTaskPtr);
	if (error != KERN_SUCCESS)
		{
		fprintf(stderr, "Tasks : call to task_by_unix_pid() failed (2).\n");
		return self;
		}
	
	if (tmpTaskPtr == (task_t)[theCell freePointer])	// Found it
		{
		tmpInt = YES;
		break;
		}
	
	if ( ! tmpProcess.p_nxt)
		break;
	getkval(kmem_file, (int)tmpProcess.p_nxt,
									sizeof(struct proc), &tmpProcess);
	}

if (tmpInt)		// Task does correspond to a process
	{
	getkval(kmem_file, (int)tmpProcess.task,
								sizeof(struct task), &tmptask);
	getkval(kmem_file, (int)tmptask.u_address,
								sizeof(struct utask), &tmputask);
	[unixProcess setStringValue:(char *)&(tmputask.uu_comm)];
	[unixPID setIntValue:tmpProcess.p_pid];
	}
else
	{
	[unixProcess setStringValue:"N/A"];
	[unixPID setStringValue:"N/A"];
	}

if (close(kmem_file) < 0)
	fprintf(stderr, "Tasks : error closing kmem_file (2).\n");

////////// Get number of threads

error = task_threads((task_t)[theCell freePointer], &theThreadArray, &tmpInt);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Tasks : error calling task_threads().\n");
	[numThreads setStringValue:"Error"];
	}
else
	{
	[numThreads setIntValue:tmpInt];
	error = vm_deallocate(task_self(), (vm_address_t) theThreadArray, 
											sizeof(theThreadArray) * tmpInt);
	if (error != KERN_SUCCESS)
        fprintf(stderr, "Tasks : trouble freeing thread list.\n");
	}

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 5;
}

@end
