// Threads.m
//
// History :
//		10.08.1992 : created by Max Tardiveau

#import "Threads.h"

@implementation Threads


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
kern_return_t		error;
thread_array_t		threadList;
unsigned int		threadCount;
int					i, cellCount;
id					newCell;
char				tmpChar[256];

if ([theCell updater])		// It's a thread
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer]];
	return self;
	}

error = task_threads((task_t)[theCell freePointer], &threadList, &threadCount);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Threads : error calling task_threads().\n");
	return self;
	}

for (i = 0; i < threadCount; i++)
	{
	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	sprintf(tmpChar, "Thread %i", cellCount);
	[newCell setStringValue:tmpChar];
	[newCell setFreePointer:(void *)threadList[i]];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	}

[theCell setMasterInstance:self];

error = vm_deallocate(task_self(), (vm_address_t)threadList, 
										sizeof(threadList) * threadCount);
if (error != KERN_SUCCESS)
        fprintf(stderr, "Threads : trouble freeing threadList.\n");


return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t				error;
struct thread_basic_info	basicInfo;
struct thread_sched_info	schedInfo;
unsigned int				basicInfo_count = THREAD_BASIC_INFO_COUNT;
unsigned int				schedInfo_count = THREAD_SCHED_INFO_COUNT;
int							i;

for (i = 0; i < 5; i++)
	[[stateMatrix cellAt:i :0] setState:0];

error = thread_info((thread_t)[theCell freePointer], THREAD_BASIC_INFO, 
								(thread_info_t)&basicInfo, &basicInfo_count);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Threads : error calling thread_info(1).\n");
	[userRunTimeSec setStringValue:"error"];
	[userRunTimeUSec setStringValue:"error"];
	[systemRunTimeSec setStringValue:"error"];
	[systemRunTimeUSec setStringValue:"error"];
	[scaledCPU setStringValue:"error"];
	[basePriority setStringValue:"error"];
	[currentPriority setStringValue:"error"];
	[suspendCount setStringValue:"error"];
	[sleepTime setStringValue:"error"];
	[swapped setState:0];
	[idle setState:0];
	}
else
	{
	[userRunTimeSec setIntValue:basicInfo.user_time.seconds];
	[userRunTimeUSec setIntValue:basicInfo.user_time.microseconds];
	[systemRunTimeSec setIntValue:basicInfo.system_time.seconds];
	[systemRunTimeUSec setIntValue:basicInfo.system_time.microseconds];
	[scaledCPU setIntValue:basicInfo.cpu_usage];
	[basePriority setIntValue:basicInfo.base_priority];
	[currentPriority setIntValue:basicInfo.cur_priority];
	[suspendCount setIntValue:basicInfo.suspend_count];
	[sleepTime setIntValue:basicInfo.sleep_time];
	[swapped setState:(basicInfo.flags & TH_FLAGS_SWAPPED)];
	[idle setState:(basicInfo.flags & TH_FLAGS_IDLE)];
	[[stateMatrix cellAt:(basicInfo.run_state - 1) :0] setState:1];
	}

for (i = 0; i < 3; i++)
	[[policyMatrix cellAt:i :0] setState:0];

error = thread_info((thread_t)[theCell freePointer], THREAD_SCHED_INFO, 
								(thread_info_t)&schedInfo, &schedInfo_count);
if (error != KERN_SUCCESS)
	{
    fprintf(stderr, "Threads : error calling thread_info(2).\n");
	[quantum setStringValue:"error"];
	[maxPriority setStringValue:"error"];
	[depressedFrom setStringValue:"error"];
	}
else
	{
	switch (schedInfo.policy)
		{
		case POLICY_FIXEDPRI :
			[[policyMatrix cellAt:0 :0] setState:1];
			break;
		case POLICY_TIMESHARE :
			[[policyMatrix cellAt:1 :0] setState:1];
			break;
		case POLICY_INTERACTIVE :
			[[policyMatrix cellAt:2 :0] setState:1];
			break;
		default :
			fprintf(stderr, "Threads : unknown policy.\n");
			break;
		}
	
	if (schedInfo.policy == POLICY_FIXEDPRI)
		[quantum setIntValue:schedInfo.data];
	else
		[quantum setStringValue:"N/A"];
		
	[maxPriority setIntValue:schedInfo.max_priority];
	if (schedInfo.depressed)
		[depressedFrom setIntValue:schedInfo.depress_priority];
	else
		[depressedFrom setStringValue:"N/A"];
	}

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 5;
}

@end
