// VMRegions.m
//
// History :
//		10.09.1992 : created by Max Tardiveau

#import "VMRegions.h"

@implementation VMRegions


////////////////////////////////////////////////////////////////
// 

- fillList:theMatrix fromCell:theCell
{
kern_return_t		error;
vm_address_t		address = VM_MIN_ADDRESS;
vm_size_t			size;
vm_prot_t			protection, max_protection;
vm_inherit_t		inheritance;
boolean_t			shared;
port_t				object_name;
vm_offset_t			offset;
int					cellCount;
id					newCell;
char				tmpChar[40];

if ([theCell updater])		// It's a VMRegion
	{
	[self fillSubList:theMatrix ofCell:theCell pointer:[theCell freePointer] 
							secPointer:[theCell secondaryPointer] string:NULL];
	return self;
	}

while (1)
	{
	error = vm_region((vm_task_t)[theCell freePointer], &(vm_address_t)address, 
				&size, &protection, &max_protection, &inheritance, &shared, 
				&object_name, &offset);

	if (error == KERN_NO_SPACE)
		break;

	if (error != KERN_SUCCESS)
		{
		fprintf(stderr, "Threads : error calling vm_region().\n");
		return self;
		}

	[theMatrix addRow];
	cellCount = [theMatrix cellCount];
	newCell = [theMatrix cellAt:(cellCount - 1) :0];
	sprintf(tmpChar, "Region %i", cellCount);
	[newCell setStringValue:tmpChar];
	[newCell setFreePointer:[theCell freePointer]];
	[newCell setSecondaryPointer:(void *)address];
	[newCell setUpdater:self];
	[newCell setDir:[theCell getDir]];
	[newCell setLeaf:NO];
	address += size;
	}

[theCell setMasterInstance:self];

return self;
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
kern_return_t		error;
vm_address_t		address;
vm_size_t			size;
vm_prot_t			protection, max_protection;
vm_inherit_t		inheritance;
boolean_t			shared;
port_t				object_name;
vm_offset_t			offset;
char				tmpChar[40];
int					i;

address = (vm_address_t)[theCell secondaryPointer];
error = vm_region((vm_task_t)[theCell freePointer],&address, &size, 
   &protection, &max_protection, &inheritance, &shared, &object_name, &offset);

if (error != KERN_SUCCESS)
	{
	[MyMaster addMessage:"VMRegions : error calling vm_region(2).\n"];
	return self;
	}

sprintf(tmpChar, "%0#8.8x", address);
[Address setStringValue:tmpChar];
[Size setIntValue:size];
sprintf(tmpChar, "%0#8.8x", offset);
[Offset setStringValue:tmpChar];

[ProtRead setState:(protection & VM_PROT_READ)];
[ProtWrite setState:(protection & VM_PROT_WRITE)];
[ProtExec setState:(protection & VM_PROT_EXECUTE)];

[MaxRead setState:(protection & VM_PROT_READ)];
[MaxWrite setState:(protection & VM_PROT_WRITE)];
[MaxExec setState:(protection & VM_PROT_EXECUTE)];

for (i = 0; i < 4; i++)
	[[InheritMatrix cellAt:i :0] setState:0];
[[InheritMatrix cellAt:inheritance :0] setState:1];

if (shared)
	[Shared setState:1];
else
	[Shared setState:0];

return self;
}

@end
