// IOstats.m
//

#import "IOstats.h"

@implementation IOstats


#define VMUNIX "/mach"
#define KMEM "/dev/kmem"

////////////////////////////////////////////////////////////////
// Get a value out of kernel memory.
// offset is the byte offset into the kernel,
// ptr points to a buffer into which the value is put,
// size is the size of the buffer (and of the object to get).

static void getkval(int kfile, unsigned long offset, int size, void *ptr)
{
if (lseek(kfile, (long)offset, 0) == -1)
	{
	fprintf(stderr, "IOstats : getkval : lseek error for %x\n", offset);
	exit(1);
	}

if (read(kfile, (char *)ptr, size) == -1)
	{
	fprintf(stderr, "IOstats : getkval : read error at %x\n", offset);
	exit(1);
	}
}

////////////////////////////////////////////////////////////////
//

- (int) getKernelValue:(char *)theString
{
struct nlist	corelist[2];

corelist[0].n_un.n_name = theString;
corelist[1].n_un.n_name = 0;

nlist(VMUNIX, corelist);
if  ((corelist[0].n_type == 0) && (corelist[0].n_value == 0))
	{
	[MyMaster addMessage:"IOstats : error in nlist.\n"];
	return 0;
	}

return corelist[0].n_value;
}


////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
int 			kmem_file;
long			tmpLongArray[MAX(DK_NDRIVE, CPUSTATES)];
float			tmpFloatArray[MAX(DK_NDRIVE, CPUSTATES)];
int				i, tmpInt;
long			tmpLong;

[theCell setUpdater:self];

if ((kmem_file = open(KMEM, 0)) < 0)
	{
	[MyMaster addMessage:"IOstats : error opening KMEM.\n"];;
	return self;
	}

getkval(kmem_file, [self getKernelValue:"_cp_time"], sizeof(long) * CPUSTATES, 
																tmpLongArray);
[CPUuser setIntValue:tmpLongArray[CP_USER]];
[CPUnice setIntValue:tmpLongArray[CP_NICE]];
[CPUsystem setIntValue:tmpLongArray[CP_SYS]];
[CPUidle setIntValue:tmpLongArray[CP_IDLE]];

getkval(kmem_file, [self getKernelValue:"_dk_ndrive"], sizeof(int), &tmpInt);
[nDrives setIntValue:tmpInt];

getkval(kmem_file, [self getKernelValue:"_dk_busy"], sizeof(int), &tmpInt);
[dkBusy setIntValue:tmpInt];

getkval(kmem_file, [self getKernelValue:"_dk_time"], sizeof(long) * DK_NDRIVE, 
																tmpLongArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:0 :i] setIntValue:tmpLongArray[i]];

getkval(kmem_file, [self getKernelValue:"_dk_seek"], sizeof(long) * DK_NDRIVE, 
																tmpLongArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:1 :i] setIntValue:tmpLongArray[i]];

getkval(kmem_file, [self getKernelValue:"_dk_xfer"], sizeof(long) * DK_NDRIVE, 
																tmpLongArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:2 :i] setIntValue:tmpLongArray[i]];

getkval(kmem_file, [self getKernelValue:"_dk_wds"], sizeof(long) * DK_NDRIVE, 
																tmpLongArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:3 :i] setIntValue:tmpLongArray[i]];

getkval(kmem_file, [self getKernelValue:"_dk_bps"], sizeof(long) * DK_NDRIVE, 
																tmpLongArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:4 :i] setIntValue:tmpLongArray[i]];

getkval(kmem_file, [self getKernelValue:"_dk_mspw"], sizeof(float) * DK_NDRIVE, 
																tmpFloatArray);
for (i = 0; i < 4; i++)
	[[diskMatrix cellAt:5 :i] setFloatValue:tmpFloatArray[i]];

getkval(kmem_file, [self getKernelValue:"_tk_nin"], sizeof(long), &tmpLong);
[termIn setIntValue:tmpLong];

getkval(kmem_file, [self getKernelValue:"_tk_nout"], sizeof(long), &tmpLong);
[termOut setIntValue:tmpLong];

return self;
}

////////////////////////////////////////////////////////////////
//

- (int) updateInterval
{
return 3;
}

@end
