// ShutdownLog.m
//

#import "ShutdownLog.h"

@implementation ShutdownLog


////////////////////////////////////////////////////////////////
//

- init
{
[super init];

myFont = [Font newFont:"Ohlfs"  size:11.0];

return self;
}

////////////////////////////////////////////////////////////////
//

static void formatDuration(long seconds, char *buffer)
{
long		minutes = 0, hours = 0, days = 0;

if (seconds >= (60*60*24))
	{
	days = seconds / (60*60*24);
	seconds %= (60*60*24);
	}
if (seconds >= (60*60))
	{
	hours = seconds / (60*60);
	seconds %= (60*60);
	}
if (seconds >= 60)
	{
	minutes = seconds / 60;
	}

if (days)
	sprintf(buffer, "%3id %2ih %2im", days, hours, minutes);
else if (hours)
	sprintf(buffer, "     %2ih %2im", hours, minutes);
else if (minutes)
	sprintf(buffer, "         %2im", minutes);
else
	sprintf(buffer, "          0m");
}

////////////////////////////////////////////////////////////////
//

- doUpdateForCell:theCell
{
FILE		*wtmpFile;
struct utmp tmpEntry;
id			myText;
char		tmpChar[256];
char		rebootTime[40], timeOnChar[30], shutdownTime[40], timeOffChar[30];
long		lastReboot = 0;
long		lastShutdown = 0;
long		timeOn, timeOff;

myText = [myScrollView docView];
[myText setText:""];

wtmpFile = fopen("/usr/adm/wtmp", "r");
if (wtmpFile == NULL)
	{
	[myText setText:"Error opening /usr/adm/wtmp"];
	return self;
	}

while (fread(&tmpEntry, sizeof(struct utmp), 1, wtmpFile))
	{
	if (tmpEntry.ut_line[0] != '~')		// we want only shutdown stuff
		continue;
	
	if ( ! lastReboot)
		{
		if ( strncmp(tmpEntry.ut_name, "reboot", 6))
			continue;
		else
			{
			lastReboot = tmpEntry.ut_time;
			continue;
			}
		}
	else
		if ( strncmp(tmpEntry.ut_name, "shutdown", 8))
			continue;
		
	timeOn = tmpEntry.ut_time - lastReboot;
	
	strcpy(rebootTime, ctime(&lastReboot));
	rebootTime[20] = rebootTime[22];
	rebootTime[21] = rebootTime[23];
	rebootTime[22] = '\0';
	
	strcpy(shutdownTime, ctime(&tmpEntry.ut_time));
	shutdownTime[20] = shutdownTime[22];
	shutdownTime[21] = shutdownTime[23];
	shutdownTime[22] = '\0';
	
	formatDuration(timeOn, timeOnChar);
	
	[myText setSel:1000000 :1000000];
	
	if (lastShutdown)
		{
		timeOff = lastReboot - lastShutdown;
		formatDuration(timeOff, timeOffChar);
		sprintf(tmpChar, " | %s\n%s | %s | %s", timeOffChar, rebootTime,
										timeOnChar, shutdownTime);
		}
	else
		sprintf(tmpChar, "%s | %s | %s", rebootTime,
										timeOnChar, shutdownTime);
	
	[myText replaceSel:tmpChar];
	
	lastShutdown = tmpEntry.ut_time;
	lastReboot = 0;
	}

if (lastReboot)			// Last line
	{
	strcpy(rebootTime, ctime(&lastReboot));
	rebootTime[20] = rebootTime[22];
	rebootTime[21] = rebootTime[23];
	rebootTime[22] = '\0';
	timeOff = lastReboot - lastShutdown;
	formatDuration(timeOff, timeOffChar);
	sprintf(tmpChar, " | %s\n%s | Still going  |", timeOffChar, rebootTime);
	[myText setSel:1000000 :1000000];
	[myText replaceSel:tmpChar];
	}

[myText setFont:myFont];

fclose(wtmpFile);

return self;
}


@end
