/* File: MagnifyView.h - View class for 'Magnify'
 *
 * By: Christopher Lane <lane@camis.stanford.edu>
 * Symbolic Systems Resources Group
 * Knowledge Systems Laboratory
 * Stanford University
 *
 * PostScript optimizations by: Scott Byer <byer@adobe.com>
 *
 * Date:  13 May 1994
 *
 * Copyright: 1990, 1991, 1992  & 1994 by The Leland Stanford Junior University.
 * This program may be distributed without restriction for non-commercial use.
 */
 
#import <mach/cthreads.h>
#import <appkit/appkit.h>

@interface MagnifyView : View
{
	int flags;
	BOOL frozen, resized;
	NXSize offset;
	NXCoord scale;
	NXPoint mouse;
	id invisibleImage, invisibleWindow, invisibleView;
	
	double updateRate;
	BOOL autoUpdate;
	DPSTimedEntry timedEntry;

	id fakeCursor;
	unsigned int showFakeCursor;

	BOOL showGrid;
	NXCoord gridSize, *gridNumArray;
	unsigned int lenGridNumArray, usedGridNumArray;
}

- initFrame:(const NXRect *) frameRect;
- (BOOL) acceptsFirstResponder;
- copy:sender;
- drawSelf:(const NXRect *) rects :(int) rectCount;
- free;
- sizeTo:(NXCoord) newWidth :(NXCoord) newHeight;
- mouseMoved:(NXEvent *) theEvent;
- flagsChanged:(NXEvent *) theEvent;

- updateImage;

- drawFakeCursor;
- drawGrid;
- resizeGrid;

- (int) flags;
- (NXPoint *) mouse;

- (NXCoord) scale;
- setScale:(NXCoord) newScale;

- (NXCoord) gridSize;
- setGridSize:(NXCoord) value;

- (float) updateRate;
- setUpdateRate:(float) value;

- (BOOL) autoUpdate;
- setAutoUpdate:(BOOL) value;

- setupTimedEntry;
- resetTimedEntry;

- (BOOL) resized;

- (BOOL) isFrozen;
- setFrozen:(BOOL) state;
- freeze;
- unfreeze;

- (BOOL) showGrid;
- setShowGrid:(BOOL) state;

- fakeCursor;
- setFakeCursor:cursor;

- (BOOL) showFakeCursor;
- setShowFakeCursor:(BOOL) state;

- createWindows;

@end
