// 
//  SCSI_Inspector.app - a SCSI reporting tool for Motorola and Intel 
//						 systems running NeXTSTEP version 3.2 or higher.
//
//  Copyright (C) 1994 by Christopher Wolf.
//  
//	This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  Bug-reports, comments and questions should be directed to:
//		Christopher Wolf <chris@alchemy.geo.cornell.edu>
//
//  Portions of this program are derived from earlier code 
//  (SCSI Inquirer) Copyright (C) 1990, 1991, 1992 by Jiro Nakamura 
//  and used with permission.
//

//
// 	For best results view this file with a tab size of 4 and
//	a display width of 132 columns or wider.
//


//	file version information
#define RCSIWH "$Id: SCSIInspectorWindow.h,v 0.30 94/09/14 23:31:13 chris Exp Locker: chris $"

//	NeXTSTEP headers
#import <appkit/Matrix.h>
#import <appkit/Panel.h>
#import <appkit/View.h>
#import <appkit/Window.h>
#import <appkit/PopUpList.h>

//	generic Unix headers
#import <sys/types.h>

//	SCSI_Inspector.app headers
#import "SCSI2_Kit.subproj/scsi2reg.h"
#import "SCSI2_Kit.subproj/SCSI.h"
#import "SCSI2_Kit.subproj/DriveSCSI.h"

@interface SCSIInspectorWindow:Window
	{

	id selector;
	id deviceNameText;
	id deviceNumText;
	id scsi2Text;
	id popupButton;
	id popupList;
	id benchmarksItem;
	
	int target, lun, lastMode;
	
	NXStream *output;	
	
	struct DeviceInfo
		{
		struct inquiry2_reply irBuffer;
		char typeString[32];
		BOOL deviceValid;
		} devInfo[7];
			
	}

- (const char *) appName;
- chooseMode: sender;
- doItAsynch: sender;

- (int) setTarget: (int) target lun: (int) lun;
- outputInfo: (DriveSCSI *)scsi;
- outputDiagnostics: (DriveSCSI *)scsi;
- outputBenchmarks: (DriveSCSI *)scsi;
- doBenchDrive: (DriveSCSI *)scsi Sector: (int)Sector BufSizes: (int *)bufSizes Iterations: (int)iterations;

- (const char *) deviceTypeDescription: (int)deviceType;
- (const char *) ansiVersionDescription: (int)ansiVersion;
- (const char *) bufModeDescription: (int)bufMode;
- (const char *) speedDescription: (int)speedCode;
- (const char *) eodDescription: (int)eod;
- (const char *) compressionDescription: (int)sdca;
- (const char *) densityCodeDescription: (int)densityCode;
- (const char *) mediumTypeDescription: (int)mt;

- (BOOL) targetIsValid;
- (BOOL) targetIsRemovable;
- (int)  targetDeviceType;
- (char *) targetVendorID;
- (char *) targetProductID;
- (char *) targetDeviceTypeString;

@end
