// 
//  SCSI_Inspector.app - a SCSI reporting tool for Motorola and Intel 
//						 systems running NeXTSTEP version 3.2 or higher.
//
//  Copyright (C) 1994 by Christopher Wolf.
//  
//	This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  Bug-reports, comments and questions should be directed to:
//		Christopher Wolf - chris@alchemy.geo.cornell.edu
//
//  Portions of this program are based on earlier code (SCSI Inquirer)
//  Copyright 1992 by Jiro Nakamura and used with permission.
//

//
// 	For best results view this file with a tab size of 4 and
//	a display width of 132 columns or wider.
//

// 	file version information
#define RCSSWM "$Id: SCSISelectorWindow.m,v 0.18 94/09/19 18:01:13 chris Exp Locker: chris $"

//	NeXTSTEP headers
#import <appkit/Application.h>
#import <appkit/TextField.h>
#import <appkit/Matrix.h>
#import <appkit/Button.h>
#import <appkit/ButtonCell.h>
#import <defaults/defaults.h>	

//	generic Unix headers
#import <strings.h>
#import <sys/types.h>
#import <libc.h>					// For geteuid()
#import <sys/vnode.h>				// for chmod code

// 	SCSI_Inspector.app headers
#import "SCSISelectorWindow.h"
#import "SCSIInspectorWindow.h"
#import "VersionNumber.h"

// string constants
const char RCSswm[] = RCSSWM;
const char RCSswh[] = RCSSWH;

@implementation SCSISelectorWindow


- appDidInit: sender
	{
	[self makeKeyAndOrderFront: self];
	return self;
	}
	
	
// 
// Delegate for NXBrowser
// 

- (int) browser: sender getNumRowsInColumn: (int) column
	{
	return 8;
	}
	
- browser: sender loadCell: cell atRow: (int)row inColumn: (int)column
	{
	
	char tempBuffer[32];
		
	if ([inspectorWindow setTarget: row lun: 0])
		{
		sprintf(tempBuffer, "%d - %.8s %.16s",  
				row, [inspectorWindow targetVendorID], [inspectorWindow targetProductID]);
		}
	else
		{
		sprintf(tempBuffer, "%d - No Device", row);
		}

	[cell setStringValue: tempBuffer];
	[cell setLeaf: YES];
	[cell setLoaded: YES];
				
	return self;

	}
	

//
// Checks to make sure we are running as root and other misc. stuff.
//

- update
	{

	char tempBuffer[64], *tbi;
	
	struct stat buf;
	const char *sendTypes[3];
	const char *returnTypes[2];
	
	[super update];

    sendTypes[0] = NXAsciiPboardType;
    sendTypes[1] = NULL;
    sendTypes[2] = NULL;
    returnTypes[0] = NXAsciiPboardType;
    returnTypes[1] = NULL;
    [NXApp registerServicesMenuSendTypes:sendTypes andReturnTypes:returnTypes];

	sprintf(tempBuffer,"%s %s",REVNUMBER,REVDATE);
    for (tbi = tempBuffer; *tbi; tbi++)
		{
		if (*tbi == '$')
			{
			*tbi = ' ';
			}
		}
		
	[versionInfoText setStringValue: tempBuffer];
	
	if(geteuid())
		{
		NXRunAlertPanel("Not running as Root", 
 			 	  		"This program must be run as root in order to access "
				 	    "SCSI devices. See manual for more information.", 
						"OK", NULL, NULL);
		exit(1);
		}
		
	stat( NXArgv[0],  &buf);
	if( !(buf.st_mode & 04000) && !(geteuid()) )
		{
		[installSelfMenuItem setEnabled:YES];
		}
	
	return self;

	}


//
// Changes program to SUID root if possible.
// 

- installSelf: sender
	{

	struct stat buf;
		
	if( !NXRunAlertPanel("Install self as SET-UID root", 
		"Installing this program as SET-UID root allows any user "
		"to run this program as root and represents a security "
		"hazard for some sites. Please read the documentation for more "
		"details.", "Install", "Cancel", NULL))
		{
		return nil;
		}
		
	stat( NXArgv[0],  &buf);
	chown( NXArgv[0], 0, 3);
	chmod( NXArgv[0], VSUID | buf.st_mode);
	fprintf( stderr, "%s: Set owner to root.bin (0.3), SET-UID root.\n", [NXApp appName]);
	
	[installSelfMenuItem setEnabled:NO];
	NXRunAlertPanel("Installation Successful", "Program installed successfully.", "OK", NULL, NULL);
		
	return self;
	
	}
	

//
// Inquire currently selected device.
//

- inquireSelected: sender
	{
	
	[inspectorWindow makeKeyAndOrderFront: self];
	if ([sender isMemberOf: [NXBrowser class]])
		{
		[inspectorWindow setTarget: [[sender matrixInColumn: [sender selectedColumn]] selectedRow] lun: 0];
		}
	[inspectorWindow update];
	return self;
	
	}


@end
